/*****************************************************************************/
/* Kernel Hooks Interface.                                    		     */
/* Version 1.4                                                               */
/* Author: Richard J Moore richardj_moore@uk.ibm.com                         */
/* 	   Vamsi Krishna S. r1vamsi@in.ibm.com 				     */
/*                                                                           */
/* A sample kernel module registering exits for hooks defined in khook.o.    */
/*                                                                           */
/* Refer to Documentation/hook/HOWTO for details.                            */
/*****************************************************************************/
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/stddef.h>
#include <linux/unistd.h>
#include <linux/hook.h>

#define TEST_HOOK1 test_hook1
#define TEST_HOOK2 test_hook2
#define TEST_HOOK3 test_hook3
#define TEST_HOOK4 test_hook4
#define TEST_HOOK5 test_hook5
int exit1(hook_rec_t *, int *);	/* first exit for test_hook1 */
int exit2(hook_rec_t *);		/* first exit for test_hook2 */
int exit3(hook_rec_t *, int *);	/* second exit for test_hook1 */
int exit4(hook_rec_t *);		/* second exit for test_hook2 */
int exit_var0(hook_rec_t *);
int exit_var1(hook_rec_t *, int);
int exit_var2(hook_rec_t *, int, int);
int exit_var2_mismatch(hook_rec_t *, int);

int ex_exit_var0(hook_rec_t *);
int ex_exit_var1(hook_rec_t *, int);
int ex_exit_var2(hook_rec_t *, int, int);
int exit_var_args(hook_rec_t * hookrec, ...);
extern int testhook(void);

hook_rec_t hook1;
hook_rec_t hook2;
hook_rec_t hook3;
hook_rec_t hook4;
hook_rec_t var0;
hook_rec_t var1;
hook_rec_t var2;
hook_rec_t ex_var0;
hook_rec_t ex_var1;
hook_rec_t ex_var2;

int init_module(void)
{
	int rc;

	hook1.hook_exit = &exit1;
	hook1.hook_exit_name = "exit1";
	rc = hook_exit_register(TEST_HOOK1, &hook1, 1);
	DPRINTK("hook_exit_register exit1 for hook1 returned %u\n", rc);
	if(rc) goto err;

	hook2.hook_exit = &exit2;
	hook2.hook_exit_name = "exit2";
	rc = hook_exit_register(TEST_HOOK2, &hook2, 1);
	DPRINTK("hook_exit_register exit2 for hook2 returned %u\n", rc);
	if(rc) goto err1;

	hook3.hook_exit = &exit3;
	hook3.hook_exit_name = "exit3";
	rc = hook_exit_register(TEST_HOOK1, &hook3, 1);
	DPRINTK("hook_exit_register exit3 for hook1 returned %u\n", rc);
	if(rc) goto err2;

	hook4.hook_exit = &exit4;
	hook4.hook_exit_name = "exit4";
	rc = hook_exit_register(TEST_HOOK2, &hook4, 0);
	DPRINTK("hook_exit_register exit4 for hook2 returned %u\n", rc);
	if(rc) goto err3;


	var0.hook_exit = &exit_var0;
	rc = hook_exit_register(TEST_HOOK3, &var0, 0);
	if(rc) goto err4;

	var1.hook_exit = &exit_var1;
	rc = hook_exit_register(TEST_HOOK4, &var1, 0);
	if(rc) goto err5;

	var2.hook_exit = &exit_var2_mismatch;
	rc = hook_exit_register(TEST_HOOK5, &var2, 0);
	if(rc) goto err6;

	ex_var0.hook_exit = &ex_exit_var0;
	rc = hook_exit_register(EX_TEST_HOOK3, &ex_var0, 0);
	if(rc) goto err7;

	ex_var1.hook_exit = &ex_exit_var1;
	rc = hook_exit_register(EX_TEST_HOOK4, &ex_var1, 0);
	if(rc) goto err8;

//	ex_var2.hook_exit = &ex_exit_var2;
	ex_var2.hook_exit = exit_var_args;
	rc = hook_exit_register(EX_TEST_HOOK5, &ex_var2, 0);
	if(rc) goto err9;

	/* now arm all exits in a single operation */
	hook1.hook_link = &hook2;
	hook2.hook_link = &hook3;
	hook3.hook_link = &hook4;
	hook4.hook_link = &var0;
	var0.hook_link = &var1;
	var1.hook_link = &var2;
	var2.hook_link = &ex_var0;
	ex_var0.hook_link = &ex_var1;
	ex_var1.hook_link = &ex_var2;
	ex_var2.hook_link = NULL;

	printk("hook exits are registered, but not yet armed.\n");

	testhook();

	rc = hook_exit_arm(&hook1);
	DPRINTK("hook_exit_arm returned %u\n", rc);
	if(rc) goto err10;

	printk("hook exit are now armed.\n");

	testhook();

	printk("disarming exit1 only\n");
	hook1.hook_link = NULL;
	rc = hook_exit_disarm(&hook1);
	DPRINTK("hook_exit_disarm returned %u\n", rc);
	if(rc) goto error;

	testhook();

	DPRINTK("disarming exit2 only\n");
	hook2.hook_link = NULL;
	rc = hook_exit_disarm(&hook2);
	DPRINTK("hook_exit_disarm returned %u\n", rc);
	if(rc) goto error;

	testhook();
	return 0;
	
error:
err10:	hook_exit_deregister(&ex_var2);
err9:	hook_exit_deregister(&ex_var1);
err8:	hook_exit_deregister(&ex_var0);
err7:	hook_exit_deregister(&var2);
err6:	hook_exit_deregister(&var1);
err5:	hook_exit_deregister(&var0);
err4:	hook_exit_deregister(&hook4);
err3:	hook_exit_deregister(&hook3);
err2:	hook_exit_deregister(&hook2);
err1:	hook_exit_deregister(&hook1);
err: 	return rc;
}

void cleanup_module(void)
{
	/* link all hook_rec structs and deregister */
	hook1.hook_link = &hook2;
	hook2.hook_link = &hook3;

	hook_exit_deregister(&hook1);

	printk("deregistered all hook exits.\n");

	testhook();

	return;
}

/* exit1 exit to test_hook1 */
int exit1(hook_rec_t * hookrec, int *rc)
{
	printk("exit1: hook exit1 entered, indicate for testhook to return immediately\n");
	*rc = 0;
	printk("exit1: hook exit1 exiting\n");
	return HOOK_RETURN;
}

/* exit2 exit to test_hook2 */
int exit2(hook_rec_t * hookrec)
{
	printk("exit2: hook exit2 entered\n");
	printk("exit2: hook exit2 exiting\n");
	return HOOK_CONTINUE;
}

/* exit3 exit to test_hook1 */
int exit3(hook_rec_t * hookrec, int *rc)
{
	printk("exit3: hook exit3 entered, allow testhook to continue\n");
	*rc = 0;
	printk("exit3: hook exit3 exiting\n");
	return HOOK_CONTINUE;
}

/* exit4 exit to test_hook2 */
int exit4(hook_rec_t * hookrec)
{
	printk("exit4: hook exit4 entered\n");
	printk("exit4: hook exit4 exiting\n");
	return HOOK_CONTINUE;
}

int exit_var0(hook_rec_t *hr)
{
	printk("exit_var0: entered\n");
	printk("exit_var0: exiting\n");
	return HOOK_CONTINUE;
}
int exit_var1(hook_rec_t *hr, int i)
{
	printk("exit_var1: entered, %x\n", i);
	printk("exit_var1: exiting\n");
	return HOOK_CONTINUE;
}
int exit_var2(hook_rec_t *hr, int i, int j)
{
	printk("exit_var2: entered %x, %x\n", i, j);
	printk("exit_var2: exiting\n");
	return HOOK_CONTINUE;
}

int exit_var2_mismatch(hook_rec_t *hr, int i)
{
	printk("exit_var2: entered using only the first arg: %x\n", i);
	printk("exit_var2: exiting\n");
	return HOOK_CONTINUE;
}

int ex_exit_var0(hook_rec_t *hr)
{
	printk("ex_exit_var0: entered %s\n", hr->hook_head->hook_id);
	printk("ex_exit_var0: exiting\n");
	return HOOK_CONTINUE;
}
int ex_exit_var1(hook_rec_t *hr, int i)
{
	printk("ex_exit_var1: entered, %x\n", i);
	printk("ex_exit_var1: exiting\n");
	return HOOK_CONTINUE;
}
int ex_exit_var2(hook_rec_t *hr, int i, int j)
{
	printk("ex_exit_var2: entered %x, %x\n", i, j);
	printk("ex_exit_var2: exiting\n");
	return HOOK_CONTINUE;
}

int exit_var_args(hook_rec_t * hookrec, ...)
{
	va_list args;
	char buf[80];
	char *fmt;
	printk("exit_var_args: entered\n");
	va_start(args, hookrec);
	fmt = va_arg(args, char *);
	vsprintf(buf, fmt, args);
	printk("vsprintf output=%s\n", buf);
	va_end(args);
	printk("exit_var_args: exiting\n");
	return HOOK_CONTINUE;
}
