#!/bin/sh
#
# Copyright 2002, Dell Computer Corporation
# gary_lerhaupt@dell.com
#
#

# Initial stuff
if [ -z "$1" ]; then
    kernel=`uname -r`
else
    kernel="$1"
fi
driver="qla2x00"
rpmversion="v6.04.02-5"
install_log="install.$rpmversion.log"
scsidir="/lib/modules/$kernel/kernel/drivers/addon/qla2200"
sourcedir="/lib/modules/$kernel/build"
mkdir -p "$scsidir"

# Define a function to invoke commands
# This is a shell function to invoke commands
# $1 -> the command to be invoked
# $2 -> the text output to the screen
# $3 -> if set to "background", outputs "."s while waiting for completion
function InvokeCommand {
    exitval=0
    rm -f .tmp_exitval
    if [ -e .tmp_exitval ]; then
	echo $"invoke_command could not run!" >$std_err
	exit 1
    fi
    echo -en "$2..." 
    if [ "$3" == background ]; then
	(eval $1; echo "exitval=$?" >> .tmp_exitval) &
	while ! [ -e .tmp_exitval ]
	  do
	  sleep 3
	  echo -en "." 
	done
	. .tmp_exitval
	rm .tmp_exitval
    else
	eval $1; exitval=$?
    fi
    if [ $exitval -gt 0 ]; then
	echo -en "(bad exit status)" 
	script_location=`dirname $0`
	echo $1 >> $script_location/.invoke_command.log
    fi
    echo -en "\n" 
    return $exitval
}

# Check that kernel-source exists
if ! [ -e $sourcedir/include ]; then
    echo -e
    echo -e "Your kernel source for kernel $kernel is not installed.  Please"
    echo -e "confirm that his kernel and its kernel-source are installed and"
    echo -e "try again."
    echo -e "The directory $sourcedir/include does not exist!"
    echo -e "Aborting..."
    echo -e
    /usr/bin/logger -t qla2x00-clariion "Installation of qla2x00 could not proceed because your kernel source for $kernel is not installed."
    exit 1
fi
 
cd /usr/src/qla2x00/

if ! [ -e $install_log ] || [ `grep -c "^$kernel$" $install_log 2>/dev/null` -eq 0 ]; then
    echo -e "Installing qla2200/qla2300 for $kernel..." 

    # Check if install_log needs to be created
    if ! [ -e $install_log ]; then
	echo -e "Creating initial /usr/src/qla2x00/$install_log..." 
	echo -e "# This file is automatically generated as part of the qla2x00" >> $install_log
	echo -e "# RPM.  build_driver.sh uses it to keep track of which kernels" >> $install_log
	echo -e "# it has installed on." >> $install_log
    fi

    # Setup correct kernel config before compile
    cd $sourcedir
    echo -e "Preparing kernel for modular driver build..." 
    InvokeCommand "make mrproper >/dev/null 2>&1" "  make mrproper" background

    # Fix Makefile up so that EXTRAVERSION matches the kernel_version extraversion
    echo -e $"  editing kernel Makefile..."
    extraversion=`echo "$kernel" | sed 's/.*-/-/'`
    sed "s/EXTRAVERSION *=.*/EXTRAVERSION = $extraversion/" Makefile 2>/dev/null > Makefile.new
    mv -f Makefile.new Makefile
    sed "s/EXTRAVERSION=.*/EXTRAVERSION = $extraversion/" Makefile 2>/dev/null > Makefile.new
    mv -f Makefile.new Makefile

    # Copy the right config
    if [ `echo $kernel | grep smp` ]; then
        InvokeCommand "cp ./configs/kernel-`echo $kernel | sed 's/-.*//'`-`uname -m`-smp.config .config" \
        "  copying kernel-`echo $kernel | sed 's/-.*//'`-`uname -m`-smp.config to $sourcedir/.config"

    elif [ `echo $kernel | grep ent` ]; then
        InvokeCommand "cp ./configs/kernel-`echo $kernel | sed 's/-.*//'`-`uname -m`-enterprise.config .config" \
        "  copying kernel-`echo $kernel | sed 's/-.*//'`-`uname -m`-enterprise.config to $sourcedir/.config"

    elif [ `echo $kernel | grep bigmem` ]; then
        InvokeCommand "cp ./configs/kernel-`echo $kernel | sed 's/-.*//'`-`uname -m`-bigmem.config .config" \
        "  copying kernel-`echo $kernel | sed 's/-.*//'`-`uname -m`-bigmem.config to $sourcedir/.config"

    elif [ `echo $kernel | grep BOOT` ]; then
        InvokeCommand "cp ./configs/kernel-`echo $kernel | sed 's/-.*//'`-i386-BOOT.config .config" \
        "  copying kernel-`echo $kernel | sed 's/-.*//'`-i386-BOOT.config to $sourcedir/.config"

    else
        InvokeCommand "cp ./configs/kernel-`echo $kernel | sed 's/-.*//'`-`uname -m`.config .config" \
        "  copying kernel-`echo $kernel | sed 's/-.*//'`-`uname -m`.config to $sourcedir/.config"
    fi

    InvokeCommand "make oldconfig >/dev/null 2>&1" "  make oldconfig" background
    InvokeCommand "make dep >/dev/null 2>&1" "  make dep" background

    # Compile the driver dependent on smp or not
    cd /usr/src/qla2x00/
    make clean >/dev/null
    if [ `echo $kernel | grep smp` ] || [ `echo $kernel | grep ent` ] || [ `echo $kernel | grep bigmem` ]; then
	InvokeCommand "make SMP=1 INCLUDEDIR=$sourcedir/include all > make.$kernel.$rpmversion.log 2>&1" \
        "Building driver: make SMP=1 INCLUDEDIR=$sourcedir/include all" background
    else
	InvokeCommand "make INCLUDEDIR=$sourcedir/include all > make.$kernel.$rpmversion.log 2>&1" \
        "Building driver: make INCLUDEDIR=$sourcedir/include all" background
    fi

    # Append kernel name to file containing list of installed on kernels
    echo $kernel >> $install_log

    # Branch differently if the make succeeded or not
    if [ -e "qla2200.o" ] && [ -e "qla2300.o" ]; then
	echo -e "Make successful.  File /usr/src/qla2x00/make.$kernel.$rpmversion.log created." 

        # Rename old driver with .orig extension
        if [ -e $scsidir/qla2200_6x.o ] && ! [ -e /usr/src/qla2x00/qla2200_6x.$kernel.*.orig ]; then
	    InvokeCommand "mv -f $scsidir/qla2200_6x.o /usr/src/qla2x00/qla2200_6x.$kernel.$rpmversion.orig" \
            "Previous driver saved as /usr/src/qla2x00/qla2200_6x.$kernel.$rpmversion.orig"

  	elif [ -e /usr/src/qla2x00/qla2200_6x.$kernel.*.orig ]; then
	    InvokeCommand "mv -f /usr/src/qla2x00/qla2200_6x.$kernel.*.orig /usr/src/qla2x00/qla2200_6x.$kernel.$rpmversion.orig 2>/dev/null" \
            "Original driver renamed to /usr/src/qla2x00/qla2200_6x.$kernel.$rpmversion.orig"
	fi
        # Rename old driver with .orig extension
        if [ -e $scsidir/qla2300_6x.o ] && ! [ -e /usr/src/qla2x00/qla2300_6x.$kernel.*.orig ]; then
	    InvokeCommand "mv -f $scsidir/qla2300_6x.o /usr/src/qla2x00/qla2300_6x.$kernel.$rpmversion.orig" \
            "Previous driver saved as /usr/src/qla2x00/qla2300_6x.$kernel.$rpmversion.orig"

  	elif [ -e /usr/src/qla2x00/qla2300_6x.$kernel.*.orig ]; then
	    InvokeCommand "mv -f /usr/src/qla2x00/qla2300_6x.$kernel.*.orig /usr/src/qla2x00/qla2300_6x.$kernel.$rpmversion.orig 2>/dev/null" \
            "Original driver renamed to /usr/src/qla2x00/qla2300_6x.$kernel.$rpmversion.orig"
	fi

        # Copy the new driver into the lib directory
	if [ -e $scsidir ]; then
	    InvokeCommand "cp qla2200.o $scsidir/qla2200_6x.o" "New driver saved as $scsidir/qla2200_6x.o"
	else 
	    echo -e "New driver could not be copied to $scsidir, could not find proper directory" 
	fi
        # Copy the new driver into the lib directory
	if [ -e $scsidir ]; then
	    InvokeCommand "cp qla2300.o $scsidir/qla2300_6x.o" "New driver saved as $scsidir/qla2300_6x.o"
	else 
	    echo -e "New driver could not be copied to $scsidir, could not find proper directory" 
	fi

        # Make clean
	make clean > /dev/null


        # Do depmod, mkinitrd
	InvokeCommand "/sbin/depmod -a -F /boot/System.map-${kernel} ${kernel}" "depmod" background
        if [ `grep -c "scsi_allow_ghost_devices" /etc/modules.conf` -eq 0 ]; then
	    echo -e "Making changes to /etc/modules.conf..."
	    cp /etc/modules.conf /etc/modules.conf-orig
	    grep -v scsi_allow_ghost_devices /etc/modules.conf | grep -v qla2x00 | grep -v qla2200 | grep -v qla2300 | grep -v qla2300_6x | grep -v qla2200_6x > /etc/modules.new
	    if [ `lspci | grep QLogic | grep -c "22[0-9]*\b"` -gt 0 ]; then
		echo "alias scsi_hostadapter96 qla2200_6x" >> /etc/modules.new	
	    fi
	    if [ `lspci | grep QLogic | grep -c "23[0-9]*\b"` -gt 0 ]; then
		echo "alias scsi_hostadapter97 qla2300_6x" >> /etc/modules.new
	    fi
	    echo "options scsi_mod scsi_allow_ghost_devices=1" >> /etc/modules.new
	    mv -f /etc/modules.new /etc/modules.conf	    
	fi
	InvokeCommand "/sbin/mkinitrd -f /boot/initrd-${kernel}.img ${kernel}" "mkinitrd" background
	if [ "$kernel" == `uname -r` ]; then
	    rmmod qla2200 2>/dev/null
	    rmmod qla2300 2>/dev/null
	    rmmod qla2x00 2>/dev/null
	    rmmod qla2200_6x 2>/dev/null
	    rmmod qla2300_6x 2>/dev/null
	fi

        # Rerun lilo if necessary
	if ! [ -e /boot/grub/grub.conf ]; then
	    InvokeCommand "/sbin/lilo" "Updating lilo"
	fi

        # Need to reboot, during future reboots it will automatically be loaded
	echo -e "\nInstallation of qla2200/qla2300 for $kernel complete."  
	/usr/bin/logger -t qla2x00-clariion "Installation of qla2x00 for $kernel successful."
	exit 0
     else
	echo "# $kernel install failed.  Remove all references to this kernel in" >> $install_log
	echo "# this file and reboot your kernel in order to attempt install again." >> $install_log
	echo -e "\nInstallation of qla2x00 for $kernel failed." 
	echo -e "Check /usr/src/qla2x00/make.$kernel.$rpmversion.log for make failures." 
	/usr/bin/logger -t qla2x00-clariion "Installation of qla2x00 for $kernel failed. Check /usr/src/qla2x00/make.$kernel.log for make failures."
	exit 1
     fi
fi
