#!/bin/bash
#
# QLogic Persistent Binding Script
# Version 2.01

# Accepts two arguments in format of x.y.z.w and compares the x,y,z portions
# of the version and if $1 < $2 -> return 1; else return 0
function CompareVersions() {
        a1=`echo $1 | perl -pe 's/([0-9]*).*/\1/g'`
        a2=`echo $1 | perl -pe 's/[0-9]*\.([0-9]*).*/\1/g'`
        a3=`echo $1 | perl -pe 's/[0-9]*\.[0-9]*\.([0-9]*).*/\1/g'`
        atotal=$(( $(( $a1<<16)) + $(($a2<<8)) + $a3 ))
        b1=`echo $2 | perl -pe 's/([0-9]*).*/\1/g'`
        b2=`echo $2 | perl -pe 's/[0-9]*\.([0-9]*).*/\1/g'` 
        b3=`echo $2 | perl -pe 's/[0-9]*\.[0-9]*\.([0-9]*).*/\1/g'`
        btotal=$(( $(( $b1<<16)) + $(($b2<<8)) + $b3 ))
        if [ $atotal -lt $btotal ]; then
                return 1
        else
                return 0
        fi
}                   

kernel=`uname -r`
script_location=`dirname $0`

# Check for 2000 character modprobe limit
modutils=`rpm -q modutils | perl -pe s/modutils-//`
CompareVersions $modutils 2.4.13
compare_versions="$?"

# Grab all the persistent binding information from /proc/scsi/qla2x00/
echo $""
echo $"Generating persistent binding information."

found_option_text=""
modified_modules_conf=""
for adapter in qla2200 qla2300 qla2x00; do
	proc_dir="/proc/scsi/$adapter"
	if [ -d "$proc_dir" ]; then
		echo $""
		echo $"Found a $adapter loaded:"
		option_text="" 
	
		# Generate option text
		echo $"  Generating option text..."
		for stub in `ls "$proc_dir" 2>/dev/null | grep -iv HbaApiNode`; do
			option_port=""
			option_target=""
			option_port=`grep scsi-qla "$proc_dir/$stub" | grep "adapter-port"`
			option_target=`grep scsi-qla "$proc_dir/$stub" | grep "target" | sed 's/target-/tgt-00/' | sed 's/=/-di-00-node=/' | sed 's/\(.*\)\;/\1\;\1\;/g' | sed 's/\(.*\)\;\(.*\)node=\(.*\)\;/\1\;\2port=\3\;/g'`
			option_text="$option_text$option_port$option_target"
		done
		option_text="options ${adapter}_6x ql2xopts="`echo $option_text | sed 's/\n//' | sed 's/ //g'`
		if [ -n "$option_text" ]; then
			found_option_text="true"
		fi

		# Check for the 2000 char limit
		if [ "$compare_version" == "1" ] && [ `echo "$option_text" | wc -c | awk '{print $1}'` -gt 1990 ]; then
			echo $""
			echo $"Warning! Warning! Warning!"
			echo -n $"Your current version of modutils: "
			rpm -q modutils
			echo $""
			echo $"Your SAN configuration data exceeds 2000 characters!"
			echo $"Due to a flaw in modutils versions < 2.4.13, you cannot have more than"
			echo $"2000 characters in an option statement in /etc/modules.conf.  Until"
			echo $"you update your modutils package to a version >= 2.4.13, you will"
			echo $"not be able to use persistent binding functionality as it will cause"
			echo $"you to be unable to load your driver at all."
			echo $""
			echo $"Aborting..."
			exit 1
		fi	

		# Don't modify modules.conf if this text is already there
		echo $"  Checking for changes..."
		if [ "`grep "options ${adapter}_6x ql2xopts" /etc/modules.conf`" == "$option_text" ]; then
			echo $"  No changes since last time.  Nothing to be done."
		else
			echo $"  Modifying /etc/modules.conf..."
			grep -v "options ${adapter}_6x ql2xopts" /etc/modules.conf > /etc/modules.new
			echo $option_text >> /etc/modules.new
			mv -f /etc/modules.new /etc/modules.conf
			modified_modules_conf="true"
		fi
	fi
done

if [ -z "$found_option_text" ]; then
	echo $""
 	echo $"Could not find any persistent binding information.  Make sure your"
  	echo $"SAN is set up and working and that your QLogic module is loaded"
	echo $"before you run this script."
	exit 1
fi

# Remake initrd if necessary
if [ -n "$modified_modules_conf" ]; then
	echo $""
	if [ -e "/boot/initrd-$kernel.img" ]; then
		echo -e "Running mkinitrd -f /boot/initrd-$kernel.img $kernel..."
		/sbin/mkinitrd -f /boot/initrd-$kernel.img $kernel
		if ! [ -e /boot/grub/grub.conf ]; then
	    		echo -e "Updating lilo..."
	   		/sbin/lilo
		fi
	else
		echo -e "You will need to remake your initrds for these changes to take effect."
	fi
fi

echo $""
echo $"Done."
