#ifndef __ASM_I386_HOOK_H
#define __ASM_I386_HOOK_H
/*****************************************************************************/
/* Kernel Hooks Interface.                                    		     */
/* Version 1.2                                                               */
/* Author: Richard J Moore richardj_moore@uk.ibm.com                         */
/* 	   Vamsi Krishna S. r1vamsi@in.ibm.com 				     */
/*                                                                           */
/* Hooks optimized for the i386 architecture.				     */
/*                                                                           */
/* Refer to Documentation/hook/HOWTO for details.                            */
/*****************************************************************************/
#include <asm/pgtable.h>

#if defined(CONFIG_HOOK) || defined(CONFIG_HOOK_MODULE)

/* define a type 0 hook: 0 parameters */
#define __HAVE_ASM_HOOK
#define HOOK(h, args...) _HOOK(h, ##args)
#define _HOOK(h, args...) __HOOK(#h, h, ##args)
#define __HOOK(h, h1, args...) \
{ \
	register int tmp; \
	__asm__ __volatile__ (".global "h"; "h":movl $0, %0":"=r"(tmp)); \
	if (unlikely(tmp)) { \
		{ \
		register hook_rec_t *ptr; \
		extern hook_head_t h1##_head; \
		for (ptr = h1##_head.hook_next; \
			!((ptr->hook_flags & HOOK_ARMED) && \
			  ((*(hook_fn_t)(ptr->hook_exit))(ptr, ##args))); \
			ptr = ptr->hook_next); \
		} \
	} \
}

/* define a conditional return hook with 0 parameters */
#define __HAVE_ASM_HOOK_RET
#define HOOK_RET(h, args...) _HOOK_RET(h, ##args)
#define _HOOK_RET(h, args...) __HOOK_RET(#h, h, ##args)
#define __HOOK_RET(h, h1, args...) \
{ \
	register int tmp; \
	__asm__ __volatile__ (".global "h"; "h":movl $0, %0":"=r"(tmp)); \
	if (unlikely(tmp)) { \
		{ \
		register hook_rec_t *ptr; \
		int rc; \
		register int dsprc; \
		extern hook_head_t h1##_head; \
		for (ptr = h1##_head.hook_next; \
			!((ptr->hook_flags & HOOK_ARMED) && \
			  (dsprc=(*(hook_fn_t)(ptr->hook_exit))(ptr, &rc, ##args))); \
			ptr=ptr->hook_next); \
		if (dsprc == HOOK_RETURN) return rc; \
		} \
	} \
}

/* define a conditional return hook with 1 parameter, no return value */
#define __HAVE_ASM_HOOK_RET_NORC
#define HOOK_RET_NORC(h, args...) _HOOK_RET_NORC(h, ##args)
#define _HOOK_RET_NORC(h, args...) __HOOK_RET_NORC(#h, h, ##args)
#define __HOOK_RET_NORC(h, h1, args...) \
{ \
	register int tmp; \
	__asm__ __volatile__ (".global "h"; "h":movl $0, %0":"=r"(tmp)); \
	if (unlikely(tmp)) { \
		{ \
		register hook_rec_t *ptr; \
		register int dsprc; \
		extern hook_head_t h1##_head; \
		for (ptr = h1##_head.hook_next; \
			!((ptr->hook_flags & HOOK_ARMED) && \
			  (dsprc=(*(hook_fn_t)(ptr->hook_exit))(ptr, ##args))); \
			ptr = ptr->hook_next); \
	 	if (dsprc == HOOK_RETURN) return; \
  		} \
	} \
}

#define __HAVE_ASM_EXCLUSIVE_HOOK
#define EXCLUSIVE_HOOK(h, args...) _EXCLUSIVE_HOOK(h, ##args)
#define _EXCLUSIVE_HOOK(h, args...) __EXCLUSIVE_HOOK(#h, h, ##args)
#define __EXCLUSIVE_HOOK(h, h1, args...) \
{ \
	register int tmp; \
	__asm__ __volatile__ (".global "h"; "h":movl $0, %0":"=r"(tmp)); \
	if (unlikely(tmp)) { \
		{ \
		extern hook_head_t h1##_head; \
		(*(hook_fn_t)(h1##_head.hook_exit))(h1##_head.hook_next, ##args); \
  		} \
	} \
}

#define __HAVE_ASM_EXCLUSIVE_HOOK_RET
#define EXCLUSIVE_HOOK_RET(h, args...) _EXCLUSIVE_HOOK_RET(h, ##args)
#define _EXCLUSIVE_HOOK_RET(h, args...) __EXCLUSIVE_HOOK_RET(#h, h, ##args)
#define __EXCLUSIVE_HOOK_RET(h, h1, args...) \
{ \
	register int tmp; \
	__asm__ __volatile__ (".global "h"; "h":movl $0, %0":"=r"(tmp)); \
	if (unlikely(tmp)) { \
		{ \
		int rc; \
		register int dsprc; \
		extern hook_head_t h1##_head; \
		dsprc = (*(hook_fn_t)(h1##_head.hook_exit))(h1##_head.hook_next, &rc, ##args); \
	 	if (dsprc == HOOK_RETURN) return rc; \
  		} \
	} \
}

/* define an exclusive conditional return hook with 1 parm, no return value */
#define __HAVE_ASM_EXCLUSIVE_HOOK_RET_NORC
#define EXCLUSIVE_HOOK_RET_NORC(h, args...) _EXCLUSIVE_HOOK_RET_NORC(h, ##args)
#define _EXCLUSIVE_HOOK_RET_NORC(h, args...) __EXCLUSIVE_HOOK_RET_NORC(#h, h, ##args)
#define __EXCLUSIVE_HOOK_RET_NORC(h, h1, args...) \
{ \
	register int tmp; \
	__asm__ __volatile__ (".global "h"; "h":movl $0, %0":"=r"(tmp)); \
	if (unlikely(tmp)) { \
		{ \
		extern hook_head_t h1##_head; \
		register int dsprc = (*(hook_fn_t)(h1##_head.hook_exit))(h1##_head.hook_next, ##args); \
	 	if (dsprc == HOOK_RETURN) return; \
  		} \
	} \
}

#endif /* CONFIG_HOOK || CONFIG_HOOK_MODULE */

/* opcodes used for relative jump at Hook_bp*/
#define JMP8_OPCODE 0xeb
#define JMP32_OPCODE 0xe9

/* offsets used for relative jump to bypass jmp dispatcher */
#define JMP_NOP 0x0
#define JMP8_BYPASS 0x2
#define JMP32_BYPASS 0x05

/*
 * Sanity check the hook location for valid instructions at hook location.
 * At hook location, we should find these instructions:
 *	b9 00 00 00 00       	mov    $0x0,%ecx
 *	85 c9                	test   %ecx,%ecx
 * We can check for mov and test instructions. As these instructions encode
 * the register name in them and the register cannot be predicted, we mask out
 * the bits corresponding to registers in the opcode before comparing.
 * 
 * mov imm to reg
 * 	1011 wreg
 *	1100 011w : 1100 0reg (or)
 * test reg1 reg2
 * 	1000 010w : 11xx xyyy (xxx = reg1, yyy = reg2)
 * 	
 * v1.4: 10/04/2001
 * Unfortunately, we can't be certain that the test instruction follows right
 * after the mov, as the compiler could move some other code in between these
 * two instructions. So, is_asm_hook checks only for "mov $0, reg" at the given
 * hook location.
 */
#define OPCODE_MOV1			0xb0
#define OPCODE_MOV1_MASK		0xf0
#define OPCODE_MOV2_1			0xc6 /* first byte */
#define OPCODE_MOV2_2			0xc0 /* second byte */
#define OPCODE_MOV2_1_MASK		0xfe
#define OPCODE_MOV2_2_MASK		0xf8
#define OPCODE_TEST_1			0x84 /* first byte */
#define OPCODE_TEST_2			0xc0 /* second byte */
#define OPCODE_TEST_1_MASK		0xfe	
#define OPCODE_TEST_2_MASK		0xc0	
		
static inline int is_asm_hook(unsigned char * phookloc)
{
	if(phookloc == NULL) return 0;
	if((phookloc[0] & OPCODE_MOV1_MASK) == OPCODE_MOV1) {
		if (*((unsigned long *)(phookloc+1)) == 0)
			return 1;
	} else if (((phookloc[0] & OPCODE_MOV2_1_MASK) == OPCODE_MOV2_1) && 
		    ((phookloc[1] & OPCODE_MOV2_2_MASK) == OPCODE_MOV2_2)) {
		if (*((unsigned long *)(phookloc+2)) == 0)
			return 1;
	}
	return 0;
}

static inline void deactivate_asm_hook(hook_head_t * phookhead)
{
	unsigned char *phook = (unsigned char *) (phookhead->hook_loc);
	phook[2] = 0;
	flush_icache_range(phook + 2, phook + 2);
	return;
}

/*
 *  we activate a hook by changing the bypass jump offset to 0 - this atomic
 *  under MP
 */
static inline void activate_asm_hook(hook_head_t * phookhead)
{
	unsigned char *phook = (unsigned char *) (phookhead->hook_loc);
	phook[2] = 1;
	flush_icache_range(phook + 2, phook + 2);
	return;
}

#endif /* __ASM_I386_HOOK_H */
