/* $Id: lkst_private.h,v 1.3 2002/03/25 19:05:13 serizawa Exp $ */
/* linux/include/asm-i386/lkst_private.h */

/************************************************************
 *
 * COPYRIGHT (C) HITACHI,LTD. 2001-2002 ALL RIGHTS RESERVED.
 * WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
 *            HITACHI CENTRAL RESEARCH LABORATORY.
 *
 * Created by T.Nakamura <tetsu@crl.hitachi.co.jp>
 *
 ************************************************************/

#ifndef _ARCH_I386_LKST_PRIVATE_H
#define _ARCH_I386_LKST_PRIVATE_H

#include <asm/lkst.h>
#include <asm/msr.h>

typedef unsigned int lkst_pointer_check_digit;

#define LKST_POINTER_CHECK_MAGIC	((lkst_pointer_check_digit)0x3c695a0f)


/**
 * atomic_read_and_inc - read and increment atomic variable
 * @v: pointer of type atomic_t
 * 
 * Atomically increments @v by 1.  Note that the guaranteed
 * useful range of an atomic_t is only 24 bits.
 */

static __inline__ int atomic_read_and_add(int i, atomic_t * v)
{
	__asm__ __volatile__(LOCK "xadd %1,%0":"+m"(v->counter), "=r"(i)
			     :"0"(v->counter), "1"(i):"memory");
	return (i);
}


/*
 * function lkst_evhandlerprim_mc()
 * A primitive part of an Event-handler. This function returns current
 * value of machine-cycle-counter.
 * 
 *  Arguments
 *    NONE
 *
 *  returns
 *    tsc:	current value of machine-cycle-counter.
 * 
 */
static __inline__ lkst_tsc_t lkst_evhandlerprim_mc(void)
{
	register lkst_tsc_t tsc;

	/* Read the Time Stamp Counter */
	rdtscll(tsc);
	return tsc;
}


/*
 * get variable "cpu_khz".
 */
#define	LKST_GET_CPU_KHZ() ((lkst_cpu_freq_t)cpu_khz)


#define LKST_CLI(flags) {	\
	__save_flags((flags));	\
	__cli();}

#define LKST_STI(flags) {	\
	__restore_flags((flags));}


#define LKST_LIMIT_ADDRESS_CHECK(buffer, size, limit)			\
	(((void *) ((char *) buffer + size)) <= limit)

#endif /* _ARCH_I386_LKST_PRIVATE_H */
