/* $Id: lkst.h,v 1.9 2002/04/25 12:48:07 indoh Exp $ */
/* linux/include/linux/lkst.h */

/************************************************************
 *
 * COPYRIGHT (C) HITACHI,LTD. 2001-2002 ALL RIGHTS RESERVED.
 * WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
 *            HITACHI CENTRAL RESEARCH LABORATORY.
 *
 * Created by K.SERIZAWA <serizawa@sdl.hitachi.co.jp>
 *
 ************************************************************/

#ifndef _LINUX_LKST_H
#define _LINUX_LKST_H

/* event types */
#include <linux/lkst_events.h>

#ifdef __KERNEL__
#include <linux/types.h>
#define _IN_LINUX_LKST_H
#include <linux/hook.h>
#undef _IN_LINUX_LKST_H
#endif				/* __KERNEL__ */
#include <linux/threads.h>
#include <linux/ioctl.h>

#include <asm/lkst.h>

/* ioctl requests */
#define LKST_IOC_MAGIC			'l'
#define LKST_IOC_TRC_STATUS		_IO(LKST_IOC_MAGIC, 1)
#define LKST_IOC_TRC_START		_IO(LKST_IOC_MAGIC, 2)
#define LKST_IOC_TRC_STOP		_IO(LKST_IOC_MAGIC, 3)

#define LKST_IOC_MASKSET_READ		_IO(LKST_IOC_MAGIC, 4)
#define LKST_IOC_MASKSET_WRITE		_IO(LKST_IOC_MAGIC, 5)
#define LKST_IOC_MASKSET_SET		_IO(LKST_IOC_MAGIC, 6)
#define LKST_IOC_MASKSET_LIST		_IO(LKST_IOC_MAGIC, 7)
#define LKST_IOC_MASKSET_DELETE		_IO(LKST_IOC_MAGIC, 8)

#define LKST_IOC_EVHANDLER_LIST		_IO(LKST_IOC_MAGIC, 9)
#define LKST_IOC_EVHANDLER_CTRL		_IO(LKST_IOC_MAGIC, 10)

#define LKST_IOC_BUFFER_READ		_IO(LKST_IOC_MAGIC, 11)
#define LKST_IOC_BUFFER_SEEK		_IO(LKST_IOC_MAGIC, 12)
#define LKST_IOC_BUFFER_CREATE		_IO(LKST_IOC_MAGIC, 13)
#define LKST_IOC_BUFFER_SHIFT		_IO(LKST_IOC_MAGIC, 14)
#define LKST_IOC_BUFFER_DELETE		_IO(LKST_IOC_MAGIC, 15)
#define LKST_IOC_BUFFER_LIST		_IO(LKST_IOC_MAGIC, 16)
#define LKST_IOC_BUFFER_RING		_IO(LKST_IOC_MAGIC, 17)
#define LKST_IOC_BUFFER_SETRMOD		_IO(LKST_IOC_MAGIC, 18)

#define LKST_IOC_ENTRY_LOG		_IO(LKST_IOC_MAGIC, 19)

/* Maskset to record no event */
#define LKST_MASKSET_ID_RNOTHING	0
/* Maskset to record all events */
#define LKST_MASKSET_ID_RALL	1
/* Maskset to record events specified when kernel compilation */
#define LKST_MASKSET_ID_RDEFAULT	2

/* LKST_MASKSET_TBL_MAX means size of maskset_table. (cf. LKST_MASKSET_TBL_LEN_MAX */
#define LKST_MASKSET_TBL_MAX	((lkst_maskset_id) 0xFF)
#define LKST_MASKSET_ID_MAX	((lkst_maskset_id) (LKST_MASKSET_TBL_MAX - 1))
#define LKST_MASKSET_ID_VOID	((lkst_maskset_id) (LKST_MASKSET_ID_MAX + 1))

#define LKST_EVHANDLER_TBL_MAX			((lkst_evhandler_id) 0xFF)
#define LKST_EVHANDLER_ID_MAX						\
	((lkst_evhandler_id) (LKST_EVHANDLER_TBL_MAX - 1))
#define LKST_EVHANDLER_ID_VOID						\
	((lkst_evhandler_id) (LKST_EVHANDLER_ID_MAX + 1))

/* Definitions of event-handler ID.
 *	EVHANDLER_ID_DMX:		for Dynamic Maskset eXchange
 *	EVHANDLER_ID_DEFAULT:		default event-handler
 *	EVHANDLER_ID_BUFFER_SHIFT:	shift buffers on buffer-overflow
 *	EVHANDLER_ID_BUFFER_SHIFT_DW:	shift buffers and Wakeup LKST Daemon
 *	EVHANDLER_ID_EXCEPTION:		for debug of exception loop
 *	EVHANDLER_ID_MASKSET_XCHG:	do dynamic maskset exchange
 */
#define LKST_EVHANDLER_ID_DMX 			((lkst_evhandler_id) 0x00)
#define LKST_EVHANDLER_ID_DEFAULT		((lkst_evhandler_id) 0x01)
#define LKST_EVHANDLER_ID_BUFFER_SHIFT		((lkst_evhandler_id) 0x02)
#define LKST_EVHANDLER_ID_BUFFER_SHIFT_DW	((lkst_evhandler_id) 0x03)
#define LKST_EVHANDLER_ID_EXCEPTION		((lkst_evhandler_id) 0x04)
#define LKST_EVHANDLER_ID_MASKSET_XCHG		((lkst_evhandler_id) 0x05)

#define LKST_EVHANDLER_ID_RESERVED		((lkst_evhandler_id) 0x1f)

#define LKST_BUFFER_TBL_MAX	((lkst_buffer_id) 0xFF)
#define LKST_BUFFER_ID_MAX	((lkst_buffer_id) (LKST_BUFFER_TBL_MAX - 1))
#define LKST_BUFFER_ID_VOID	((lkst_buffer_id) (LKST_BUFFER_ID_MAX + 1))

/* Max number of CPUs */
#define LKST_CPU_MAX	32

typedef u_int64_t lkst_arg_t;
#define LKST_ARG32(high,low)						\
	((lkst_arg_t) (((lkst_arg_t) (unsigned int) high << 32) |	\
		           (unsigned int) low))

typedef unsigned char lkst_maskset_id;
typedef unsigned char lkst_buffer_id;

struct lkst_status_param {
	lkst_maskset_id	current_maskset_id;
				/* current selected maskset ID */
	lkst_buffer_id	current_buffer_id[LKST_CPU_MAX];
				/* current selected buffer ID */
	int	maskset_num;	/* total number of registered masksets */
	int	evhandler_num;	/* total number of registered event handlers */
	int	buffer_num;	/* total number of created event buffers */
};

struct lkst_entry_args {
	int event_type;		/* corresponding event type */	
	lkst_arg_t arg1;	/* 1st argument acquired at a trace point */
	lkst_arg_t arg2;	/* 2nd argument acquired at a trace point */
	lkst_arg_t arg3;	/* 3rd argument acquired at a trace point */
	lkst_arg_t arg4;	/* 4th argument acquired at a trace point */
};

				/* Array of mnemonics of event types */
extern char *lkst_event_type_mnemonics[];
				/* Array of descriptions of event types */
extern char *lkst_event_type_descriptions[];

extern void lkst_entry_log(void *phookrec, int event_type,
			   lkst_arg_t arg1,
			   lkst_arg_t arg2,
			   lkst_arg_t arg3,
			   lkst_arg_t arg4);

extern void lkst_trc_init_stage0(void);

extern void lkst_trc_init_stage1(void);

extern void lkst_trc_init_main(void);

extern void lkst_time_set(int from_settimeofday);

extern int lkst_trc_status(struct lkst_status_param *param);

extern int lkst_trc_start(void);

extern int lkst_trc_stop(void);

extern int lkst_maskset_xchg(lkst_maskset_id * id);

extern __inline__ void lkst_dummy_vmalloc(void);

static inline void * lkst_current_pc(void)
{
	__label__ here;
here:
	return (&&here);
}

				/* For IO trace */
#if (!defined(CONFIG_DEBUG_KERNEL_STATE_TRACE) && !defined(CONFIG_DEBUG_KERNEL_STATE_TRACE_MODULE)) || defined(CONFIG_HOOK) || defined(CONFIG_HOOK_MODULE) 
#define LKST_SUPPRESS_PIO_TRACE
#endif /* if (!defined(CONFIG_DEBUG_KERNEL_STATE_TRACE) && !defined(CONFIG_DEBUG_KERNEL_STATE_TRACE_MODULE)) || defined(CONFIG_HOOK) || defined(CONFIG_HOOK_MODULE)  */

				/* For GKHI */
#ifdef __KERNEL__

/* HOOK Information table */
struct lkst_hook_info {
	char* hook_id;
	hook_rec_t* phookrec;
};
extern struct lkst_hook_info lkst_hook_info_table[];
extern void lkst_hook_init_module(lkst_event_type etype);
extern void lkst_hook_reset_all(void);

#if defined(CONFIG_DEBUG_KERNEL_STATE_TRACE) ||  defined(CONFIG_DEBUG_KERNEL_STATE_TRACE_MODULE)
/* Declaration of hook header. compatible for GKHI. */
#define DECLARE_GENERIC_EXCLUSIVE_HOOK_HEAD(hk) \
char hk ## _id[sizeof(#hk)] = #hk; \
extern void hk; \
hook_head_t hk ## _head = { \
    hook_head: &(hk##_head), \
    hook_loc: NULL, \
    hook_id: hk##_id, \
    hook_flags: HOOK_HEAD_EXCLUSIVE \
}; \
EXPORT_SYMBOL_NOVERS(hk ## _head); \
extern hook_head_t hk ## _head

#if defined(CONFIG_DEBUG_KERNEL_STATE_TRACE_ENABLE_INLINE)
#define GENERIC_EXCLUSIVE_HOOK(h,args...) _GENERIC_EXCLUSIVE_HOOK(h,##args)
#define _GENERIC_EXCLUSIVE_HOOK(h,args...) __GENERIC_EXCLUSIVE_HOOK(#h,h,##args)
#define __GENERIC_EXCLUSIVE_HOOK(h, h1, args...) { \
	extern hook_head_t h1##_head; \
	if (unlikely(h1##_head.hook_flags & HOOK_HEAD_ACTIVE)) { \
		typedef int (*f)(hook_rec_t *, ...); \
		(*(hook_fn_t)(h1##_head.hook_exit))(h1##_head.hook_next, ##args); \
	} \
}
#else
#define GENERIC_EXCLUSIVE_HOOK(h,args...) 
#endif

/* HOOK Interface */
#define LKST_HOOK(etype, arg1, arg2, arg3, arg4) EXCLUSIVE_HOOK(etype##_HEADER, etype, arg1, arg2, arg3, arg4)
#if defined(CONFIG_DEBUG_KERNEL_STATE_TRACE_ENABLE_INLINE)
#define LKST_HOOK_INLINE(etype, arg1, arg2, arg3, arg4) GENERIC_EXCLUSIVE_HOOK(etype##_HEADER, etype, arg1, arg2, arg3, arg4)
#else
#define LKST_HOOK_INLINE(etype, arg1, arg2, arg3, arg4) 
#endif

/* Declaration of hook header */
#define LKST_HOOK_DECLARE(etype) \
	DECLARE_EXCLUSIVE_HOOK_HEAD(etype##_HEADER);	\
	hook_rec_t etype##_hook_rec;


#if defined(CONFIG_DEBUG_KERNEL_STATE_TRACE_ENABLE_INLINE)
#define LKST_HOOK_DECLARE_INLINE(etype) \
	DECLARE_GENERIC_EXCLUSIVE_HOOK_HEAD(etype##_HEADER);	\
	hook_rec_t etype##_hook_rec;
#else
#define LKST_HOOK_DECLARE_INLINE(etype) 
#endif
/* Initialization of hook header */
#define LKST_HOOK_INIT(etype, ret) \
	_LKST_HOOK_INIT(etype, etype##_HEADER, etype##_hook_rec, ret)

#define _LKST_HOOK_INIT(etype, hookid, hookrec, ret) ({ \
	ret = hook_initialise(hookid);                      \
	lkst_hook_info_table[etype].hook_id = #hookid;      \
	lkst_hook_info_table[etype].phookrec = &hookrec;    \
	memset(&hookrec, 0, sizeof(hook_rec_t));            \
})

/* Cleanup of hook header */
#define LKST_HOOK_TERMINATE(etype, ret) \
	_LKST_HOOK_TERMINATE(etype, etype##_HEADER, etype##_hook_rec, ret)

#define _LKST_HOOK_TERMINATE(etype, hookid, hookrec, ret) ({  \
	hook_exit_deregister(&hookrec);                           \
	ret = hook_terminate(hookid, 0);                          \
	lkst_hook_info_table[etype].hook_id = NULL;               \
	lkst_hook_info_table[etype].phookrec = NULL;              \
	memset(&hookrec, 0, sizeof(hook_rec_t));                  \
})

/* Initialization of hook header for module */
#define LKST_HOOK_INIT_MODULE(etype, ret) \
	_LKST_HOOK_INIT_MODULE(etype, etype##_HEADER, etype##_hook_rec, ret)

#define _LKST_HOOK_INIT_MODULE(etype, hookid, hookrec, ret) ({ \
	ret = hook_initialise(hookid);                             \
	lkst_hook_info_table[etype].hook_id = #hookid;             \
	lkst_hook_info_table[etype].phookrec = &hookrec;           \
	lkst_hook_init_module(etype);                              \
})

/* Cleanup of hook header for module */
#define LKST_HOOK_TERMINATE_MODULE(etype, ret) \
	_LKST_HOOK_TERMINATE(etype, etype##_HEADER, etype##_hook_rec, ret)

#else  /* !CONFIG_DEBUG_KERNEL_STATE_TRACE && !CONFIG_DEBUG_KERNEL_STATE_TRACE_MODULE */
/* dummy macros when LKST is not compiled in */
#define LKST_HOOK(etype, arg1, arg2, arg3, arg4)
#define LKST_HOOK_INLINE(etype, arg1, arg2, arg3, arg4)
#define LKST_HOOK_DECLARE(etype)
#define LKST_HOOK_DECLARE_INLINE(etype)
#define LKST_HOOK_INIT(etype, ret)
#define LKST_HOOK_TERMINATE(etype, ret)
#endif /* CONFIG_DEBUG_KERNEL_STATE_TRACE || CONFIG_DEBUG_KERNEL_STATE_TRACE_MODULE */

#else  /* __KERNEL__ */
/* dummy macros when user application is compiled in */
#define LKST_HOOK(etype, arg1, arg2, arg3, arg4) 
#define LKST_HOOK_INLINE(etype, arg1, arg2, arg3, arg4) 
#define LKST_HOOK_DECLARE(etype) 
#define LKST_HOOK_DECLARE_INLINE(etype) 
#define LKST_HOOK_INIT(etype, ret) 
#define LKST_HOOK_TERMINATE(etype, ret) 
#endif /* __KERNEL__ */

#endif				/* _LINUX_LKST_H */
