/* $Id: lkst_events.h,v 1.8 2002/06/28 11:27:56 indoh Exp $ */
/* linux/include/linux/lkst_events.h */

/************************************************************
 *
 * COPYRIGHT (C) HITACHI,LTD. 2002 ALL RIGHTS RESERVED.
 * WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
 *            HITACHI CENTRAL RESEARCH LABORATORY.
 *
 * Created by K.SERIZAWA <serizawa@sdl.hitachi.co.jp>
 *
 ************************************************************/

/*******************************************************
 *
 *  This file may be included twice or more!
 *
 *******************************************************/

	/* The list of mnemonics:
		If LKST_ETYPE_MNEMONICS is defined,
 		the list of LKST_ETYPE_DEF() will become a definition of 
 		lkst_event_type_mnemonics, 
		that is an array of mnemonics of event types.	*/
#if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DEF_MNEMONICS)&&(!defined LKST_ETYPE_DEF_MNEMONICS_DEFINED))

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the array is a pair of 
				   mnemonic and value of event type.	*/
#define LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description, event_name, filter, arg_1, arg_2, arg_3, arg_4)         \
        [(event_type)] = #mnemonic,

				/* The header of the array. */
#define LKST_ETYPE_DEF_ENUM_BEGIN       char *lkst_event_type_mnemonics[] = {

				/* The tail of the enum. It must end with 
				   LKST_ETYPE_MAX.                    	*/
#define LKST_ETYPE_DEF_ENUM_END         [LKST_ETYPE_MAX] = "LKST_ETYPE_MAX" };

				/* Stopper of multi definition. */
#define LKST_ETYPE_DEF_MNEMONICS_DEFINED

#endif /* if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DEF_MNEMONICS)&&(!defined LKST_ETYPE_DEF_MNEMONICS_DEFINED)) */

/*******************************************************/


	/* The list of descriptions:
		If LKST_ETYPE_DEF_DESCRIPTIONS is defined,
 		the list of LKST_ETYPE_DEF() will become a definition of 
 		lkst_event_type_descriptions, 
		that is an array of descriptions of event types.	*/
#if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DEF_DESCRIPTIONS)&&(!defined LKST_ETYPE_DEF_DESCRIPTIONS_DEFINED))

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the array is a pair of 
				   description and value of event type.	*/
#define LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description, event_name, filter, arg_1, arg_2, arg_3, arg_4)         \
        [(event_type)] = #description,

				/* The header of the array. */
#define LKST_ETYPE_DEF_ENUM_BEGIN       char *lkst_event_type_descriptions[] = {

				/* The tail of the enum. It must end with 
				   LKST_ETYPE_MAX.                    	*/
#define LKST_ETYPE_DEF_ENUM_END         [LKST_ETYPE_MAX] = "LKST_ETYPE_MAX" };

				/* Stopper of multi definition. */
#define LKST_ETYPE_DEF_DESCRIPTIONS_DEFINED

#endif /* if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DEF_DESCRIPTIONS)&&(!defined LKST_ETYPE_DEF_DESCRIPTIONS_DEFINED)) */

/*******************************************************/


	/* The list of event data */
#if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DEF_DATATABLE)&&(!defined LKST_ETYPE_DEF_DATATABLE_DEFINED))

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the array is a pair of 
				   description and value of event type.	*/

#define LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description, event_name, filter, arg_1, arg_2, arg_3, arg_4)         \
        [(event_type)] = { event_name, filter, { arg_1, arg_2, arg_3, arg_4 }, },

				/* The header of the array. */
#define LKST_ETYPE_DEF_ENUM_BEGIN       	struct LKST_list {                      \
                                                        char*	LKST_event_name;    \
                                                        int	LKST_filter;            \
                                                        char*	LKST_inf_args[4];   \
                                                        }LKST_event_data[]={

				/* The tail of the enum. It must end with 
				   LKST_ETYPE_MAX.                    	*/
#define LKST_ETYPE_DEF_ENUM_END         [LKST_ETYPE_MAX] = "LKST_ETYPE_MAX" };

				/* Stopper of multi definition. */
#define LKST_ETYPE_DEF_DATATABLE_DEFINED

#endif /* if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DEF_DESCRIPTIONS)&&(!defined LKST_ETYPE_DEF_DESCRIPTIONS_DEFINED)) */

/*******************************************************/


	/* The list of GKHI Declaration of hook header :
		If LKST_ETYPE_DECLARE_HOOKHEADER is defined,
 		the list of LKST_ETYPE_DEF() will become a declaration of 
 		hook header and hook_rec_t data.	*/
#if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DECLARE_HOOKHEADER)&&(!defined LKST_ETYPE_DECLARE_HOOKHEADER_DEFINED))

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the array is a pair of 
				   description and value of event type.	*/

#define LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description, event_name, filter, arg_1, arg_2, arg_3, arg_4) \
        HOOK_DECL_##hooktype(LKST_ETYPE_##mnemonic);

#define HOOK_DECL_NORMAL(etype) LKST_HOOK_DECLARE(etype)
#if defined(CONFIG_DEBUG_KERNEL_STATE_TRACE_ENABLE_INLINE)
#define HOOK_DECL_INLINE(etype) LKST_HOOK_DECLARE_INLINE(etype)
#else
#define HOOK_DECL_INLINE(etype) 
#endif
#define HOOK_DECL_MODULE(etype)
#define HOOK_DECL_UNDEF(etype)

#define LKST_ETYPE_DEF_ENUM_BEGIN
#define LKST_ETYPE_DEF_ENUM_END

				/* Stopper of multi definition. */
#define LKST_ETYPE_DECLARE_HOOKHEADER_DEFINED

#endif /* if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DECLARE_HOOKHEADER)&&(!defined LKST_ETYPE_DECLARE_HOOKHEADER_DEFINED)) */

/*******************************************************/


	/* The list of GKHI Initialization of hook header :
		If LKST_ETYPE_INIT_HOOK is defined,
 		the list of LKST_ETYPE_DEF() will become a initialization of 
 		hook.	*/
#if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_INIT_HOOK)&&(!defined LKST_ETYPE_INIT_HOOK_DEFINED))

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the array is a pair of 
				   description and value of event type.	*/

#define LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description, event_name, filter, arg_1, arg_2, arg_3, arg_4) \
        HOOK_INIT_##hooktype(LKST_ETYPE_##mnemonic)

#define HOOK_INIT_NORMAL(etype) LKST_HOOK_INIT(etype, ret); \
                                retall += ret;
#if defined(CONFIG_DEBUG_KERNEL_STATE_TRACE_ENABLE_INLINE)
#define HOOK_INIT_INLINE(etype) LKST_HOOK_INIT(etype, ret); \
                                retall += ret;
#else
#define HOOK_INIT_INLINE(etype) 
#endif
#define HOOK_INIT_MODULE(etype)
#define HOOK_INIT_UNDEF(etype)


#define LKST_ETYPE_DEF_ENUM_BEGIN
#define LKST_ETYPE_DEF_ENUM_END

				/* Stopper of multi definition. */
#define LKST_ETYPE_INIT_HOOK_DEFINED

#endif /* if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_INIT_HOOK)&&(!defined LKST_ETYPE_INIT_HOOK_DEFINED)) */

/*******************************************************/


	/* The list of GKHI Cleanup of hook header :
		If LKST_ETYPE_TERMINATE_HOOK is defined,
 		the list of LKST_ETYPE_DEF() will become a cleanup of 
 		hook.	*/
#if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_TERMINATE_HOOK)&&(!defined LKST_ETYPE_TERMINATE_HOOK_DEFINED))

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the array is a pair of 
				   description and value of event type.	*/

#define LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description, event_name, filter, arg_1, arg_2, arg_3, arg_4) \
        HOOK_TERMINATE_##hooktype(LKST_ETYPE_##mnemonic)

#define HOOK_TERMINATE_NORMAL(etype) LKST_HOOK_TERMINATE(etype, ret); \
                                     retall += ret;
#if defined(CONFIG_DEBUG_KERNEL_STATE_TRACE_ENABLE_INLINE)
#define HOOK_TERMINATE_INLINE(etype) LKST_HOOK_TERMINATE(etype, ret); \
                                     retall += ret;
#else
#define HOOK_TERMINATE_INLINE(etype) 
#endif
#define HOOK_TERMINATE_MODULE(etype)
#define HOOK_TERMINATE_UNDEF(etype)

#define LKST_ETYPE_DEF_ENUM_BEGIN
#define LKST_ETYPE_DEF_ENUM_END

				/* Stopper of multi definition. */
#define LKST_ETYPE_TERMINATE_HOOK_DEFINED

#endif /* if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_TERMINATE_HOOK)&&(!defined LKST_ETYPE_TERMINATE_HOOK_DEFINED)) */

/*******************************************************/


	/* The enumerate of event types:
		By default,
 		the list of LKST_ETYPE_DEF() will become a definition of 
 		lkst_event_type_descriptions, 
		that is an array of descriptions of event types.	*/
#if	(!(defined LKST_ETYPE_DEF))
				/* The list of LKST_ETYPE_DEF()
				   will be a definition of an enum. of
				   event types.				*/

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}.		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the enum. is a pair of 
				   a name and value of event type.   	*/

#define	LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description, event_name, filter, arg_1, arg_2, arg_3, arg_4)		\
	LKST_ETYPE_##mnemonic =	(event_type),

				/* The header of the enum. */
#define LKST_ETYPE_DEF_ENUM_BEGIN					\
	typedef enum {

				/* The tail of the enum. It must end with 
				   LKST_ETYPE_MAX.                    	*/
#define LKST_ETYPE_DEF_ENUM_END						\
		LKST_ETYPE_MAX = 0xFFF+0				\
	} lkst_event_type;

				/* remenber not redefine LKST_ETYPE_MAX. */
#define	LKST_ETYPE_MAX_IS_ENUM

#endif /* if	(!(defined LKST_ETYPE_DEF)) */


/*******************************************************/

    /* header */
LKST_ETYPE_DEF_ENUM_BEGIN

    /* Body of definitions of event types */
    /* Syntax:
        LKST_ETYPE_DEF(
            VALUE_OF_THE_EVENT_TYPE:(0x001..0xffe),
            PRIORITY_OF_THE_EVENT_TYPE:(0x00..0xff),
            TYPE OF GKHI HOOK HEADER,
            MNEMONIC_OF_THE_EVENT_TYPE,
            DESCRIPTION_OF_THE_EVENT_TYPE,
            EVENT_NAME_STRING,
            DISPLAY_FILTER,
            EVENT_ARG1,
            EVENT_ARG2,
            EVENT_ARG3,
            EVENT_ARG4 )                  */

LKST_ETYPE_DEF(0x001, 0x00, NORMAL, PROCESS_CONTEXTSWITCH, PROCESS_CONTEXTSWITCH,   \
        // 0x001 event
        "context_switch",                                                           \
         0,                                                                         \
        "pointer to task_struct(prev)",                                             \
        "pointer to task_struct(next)",                                             \
        "process state",
    \
        "process count")
LKST_ETYPE_DEF(0x002, 0x00, INLINE, PROCESS_WAKEUP, PROCESS_WAKEUP,                 \
        // 0x002 event
        "process_wakeup",                                                           \
         0,                                                                         \
        "pointer to wakeup-process(p)",                                             \
        "synchronous",                                                              \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x003, 0x00, NORMAL, PROCESS_SIGSEND, PROCESS_SIGSEND,               \
        // 0x003 event
        "process_sigsend",                                                          \
         0,                                                                         \
        "signal number(sig)",                                                       \
        "pointer to process to which signal is sent",                               \
        "pointer to info structure(info)",                                          \
         NULL)
LKST_ETYPE_DEF(0x004, 0x00, NORMAL, PROCESS_LTHREADGEN, PROCESS_LTHREADGEN,         \
        // 0x004 event
        "kernel_thread",                                                            \
         0,                                                                         \
        "pointer to kernel thread(fn)",                                             \
        "argument of kernel thread(arg)",                                           \
        "flag",                                                                     \
         NULL)

LKST_ETYPE_DEF(0x010, 0x01, NORMAL, INT_HARDWARE_ENTRY, INT_HARDWARE_ENTRY,         \
        //0x010 event
        "interrupt_hardware_entry",                                                 \
         0,                                                                         \
        "IRQ number(irq)",                                                          \
        "interrupt status(status)",                                                 \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x012, 0x01, NORMAL, INT_TASKLETHI_ENTRY, INT_TASKLETHI_ENTRY,       \
        //0x012 event
        "interrupt_tasklethi_entry",                                                \
         0,                                                                         \
        "action address(t->func)",                                                  \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x014, 0x01, NORMAL, INT_TASKLET_ENTRY, INT_TASKLET_ENTRY,           \
        //0x014 event
        "interrupt_tasklet_entry",                                                  \
         0,                                                                         \
        "action address(t->func)",                                                  \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x016, 0x01, NORMAL, INT_BH_ENTRY, INT_BH_ENTRY,                     \
        //0x016 event
        "interrupt_bh_entry",                                                       \
         0,                                                                         \
        "interrupt type(nr)",                                                       \
        "action address(bh_base)",                                                  \
         NULL,                                                                      \
         NULL)

LKST_ETYPE_DEF(0x020, 0x02, INLINE, EXCEPTION_ENTRY, EXCEPTION_ENTRY,               \
        //0x020 event
        "exception_entry",                                                          \
         0,                                                                         \
        "exception-handler address(edi)",                                           \
        "error code(esi)",                                                          \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x021, 0x02, INLINE, EXCEPTION_EXIT, EXCEPTION_EXIT,                 \
        //0x021 event
        "exception_exit",                                                           \
         0,                                                                         \
        "exception-handler address(edi)",                                           \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)

LKST_ETYPE_DEF(0x030, 0x03, NORMAL, SYSCALL_ENTRY, SYSCALL_ENTRY,                   \
        //0x030 event
        "system_call_entry",                                                        \
         0,                                                                         \
        "call number",                                                              \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x031, 0x03, NORMAL, SYSCALL_EXIT, SYSCALL_EXIT,                     \
        //0x031 event
        "system_call_exit",                                                         \
         0,                                                                         \
        "call number",                                                              \
        "return value(errno)",                                                      \
         NULL,                                                                      \
         NULL)

LKST_ETYPE_DEF(0x040, 0x04, NORMAL, FS_DEVRW, FS_DEVRW,                             \
        //0x040 event
        "fs_devrw",                                                                 \
         0,                                                                         \
        "buffer(bh)",                                                               \
        "READ/WRITE(rw)",                                                           \
        "the number of block transfered(nr)",                                       \
         NULL)
LKST_ETYPE_DEF(0x041, 0x04, NORMAL, FS_DEVEND, FS_DEVEND,                           \
        //0x041 event
        "fs_devend",                                                                \
         0,                                                                         \
        "buffer(bf)",                                                               \
        "uptodate",                                                                 \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x042, 0x04, NORMAL, FS_BUFBUSY, FS_BUFBUSY,                         \
        //0x042 event
        "fs_devwait",                                                               \
         0,                                                                         \
        "buffer(bh)",                                                               \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)

LKST_ETYPE_DEF(0x050, 0x05, INLINE, MEM_SWAPOUT, MEM_SWAPOUT,                       \
        //0x050 event
        "mem_swapout",                                                              \
         0,                                                                         \
        "pointer to page swapped-out",                                              \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x051, 0x05, NORMAL, MEM_SWAPIN, MEM_SWAPIN,                         \
        //0x051 event
        "mem_swapin",                                                               \
         0,                                                                         \
        "pointer to page swapped-in",                                               \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x052, 0x05, NORMAL, MEM_DO_NOPAGE, MEM_DO_NOPAGE,                   \
        //0x052 event
        "mem_do_nopage",                                                            \
         0,                                                                         \
        "pointer to page allocated",                                                \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x053, 0x05, NORMAL, MEM_DO_WPPAGE, MEM_DO_WPPAGE,                   \
        //0x053 event
        "mem_do_wppage",                                                            \
         0,                                                                         \
        "pointer to page",                                                          \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x054, 0x05, NORMAL, MEM_WAIT_PAGE, MEM_WAIT_PAGE,                   \
        //0x054 event
        "mem_wait_page",                                                            \
         0,                                                                         \
        "pointer to page",                                                          \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x055, 0x05, NORMAL, MEM_GET_FREEPAGE, MEM_GET_FREEPAGE,             \
        //0x055 event
        "mem_get_freepage",                                                         \
         0,                                                                         \
        "pointer to page",                                                          \
        "type of page(gfp_mask)",                                                   \
        "the number of page(order)",                                                \
        "call address")
LKST_ETYPE_DEF(0x056, 0x05, NORMAL, MEM_GET_ZEROPAGE, MEM_GET_ZEROPAGE,             \
        //0x056 event
        "mem_get_zeropage",                                                         \
         0,                                                                         \
        "pointer to page",                                                          \
        "type of page(gfp_mask)",                                                   \
        "call address",                                                             \
         NULL)
LKST_ETYPE_DEF(0x057, 0x05, NORMAL, MEM_FREEPAGE, MEM_FREEPAGE,                     \
        //0x057 event
        "mem_freepage",                                                             \
         0,                                                                         \
        "pointer to page",                                                          \
        "the number of page(order)",                                                \
        "call address",                                                             \
         NULL)
LKST_ETYPE_DEF(0x058, 0x05, INLINE, MEM_VMALLOC, MEM_VMALLOC,                       \
        //0x058 event
        "mem_vmalloc",                                                              \
         0,                                                                         \
        "address(addr)",                                                            \
        "size",                                                                     \
        "call address",                                                             \
         NULL)
LKST_ETYPE_DEF(0x059, 0x05, NORMAL, MEM_VFREE, MEM_VFREE,                           \
        //0x059 event
        "mem_vfree",                                                                \
         0,                                                                         \
        "address(addr)",                                                            \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x05a, 0x05, NORMAL, MEM_CACHE_CREATE, MEM_CACHE_CREATE,             \
        //0x05a event
        "mem_cache_create",                                                         \
         0,                                                                         \
        "name",                                                                     \
        "size",                                                                     \
        "cachep",                                                                   \
         NULL)
LKST_ETYPE_DEF(0x05b, 0x05, NORMAL, MEM_CACHE_ALLOC, MEM_CACHE_ALLOC,               \
        //0x05b event
        "mem_cache_alloc",                                                          \
         0,                                                                         \
        "cachep",                                                                   \
        "flags",                                                                    \
        "objp",                                                                     \
        "call address")
LKST_ETYPE_DEF(0x05c, 0x05, NORMAL, MEM_MALLOC, MEM_MALLOC,                         \
        //0x05c event
        "mem_malloc",                                                               \
         0,                                                                         \
        "size",                                                                     \
        "flags",                                                                    \
        "objp",                                                                     \
        "call address")
LKST_ETYPE_DEF(0x05d, 0x05, NORMAL, MEM_CACHE_FREE, MEM_CACHE_FREE,                 \
        //0x05d event
        "mem_cache_free",                                                           \
         0,                                                                         \
        "cachep",                                                                   \
        "objp",                                                                     \
        "call address",                                                             \
         NULL)
LKST_ETYPE_DEF(0x05e, 0x05, NORMAL, MEM_FREE, MEM_FREE,                             \
        //0x05e event
        "mem_free",                                                                 \
         0,                                                                         \
        "objp",                                                                     \
        "call address",                                                             \
         NULL,                                                                      \
         NULL)

LKST_ETYPE_DEF(0x060, 0x06, NORMAL, NET_PKTSEND, NET_PKTSEND,                       \
        //0x060 event
        "net_pktsend",                                                              \
         0,                                                                         \
        "skb",                                                                      \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x061, 0x06, NORMAL, NET_PKTSENDI, NET_PKTSENDI,                     \
        //0x061 event
        "net_pktsendi",                                                             \
         0,                                                                         \
        "h",                                                                        \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x062, 0x06, NORMAL, NET_PKTRECV, NET_PKTRECV,                       \
        //0x062 event
        "net_pktrecv",                                                              \
         0,                                                                         \
        "skb",                                                                      \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x063, 0x06, NORMAL, NET_PKTRECVI, NET_PKTRECVI,                     \
        //0x063 event
        "net_pktrecvi",                                                             \
         0,                                                                         \
        "h",                                                                        \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x064, 0x06, NORMAL, NET_SOCKETIF, NET_SOCKETIF,                     \
        //0x064 event
        "net_socketif",                                                             \
         0,                                                                         \
        "call",                                                                     \
        "args",                                                                     \
         NULL,                                                                      \
         NULL)

LKST_ETYPE_DEF(0x070, 0x07, NORMAL, SYSV_IPC, SYSV_IPC,                             \
        //0x070 event
        "sysv_ipc",                                                                 \
         0,                                                                         \
        "call/first",                                                               \
        "second/third",                                                             \
        "*ptr",                                                                     \
         NULL)

    /* Events concerned with spinlocks should have event_type as 0x08x. */
LKST_ETYPE_DEF(0x080, 0x08, INLINE, LK_SPINLOCK, LK_SPINLOCK,                       \
        //0x080 event
        "spin_lock",                                                                \
         0,                                                                         \
        "call address",                                                             \
        "lock",                                                                     \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x081, 0x08, INLINE, LK_SPINTRYLOCK, LK_SPINTRYLOCK,                 \
        //0x081 event
        "spin_trylock",                                                             \
         0,                                                                         \
        "call address",                                                             \
        "lock",                                                                     \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x082, 0x08, INLINE, LK_SPINUNLOCK, LK_SPINUNLOCK,                   \
        //0x082 event
        "spin_unlock",                                                              \
         0,                                                                         \
        "call address",                                                             \
        "lock",                                                                     \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x083, 0x08, INLINE, LK_WRLOCK, LK_WRLOCK,                           \
        //0x083 event
        "write_lock",                                                               \
         0,                                                                         \
        "call address",                                                             \
        "rwlock",                                                                   \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x084, 0x08, INLINE, LK_WRTRYLOCK, LK_WRTRYLOCK,                     \
        //0x084 event
        "write_trylock",                                                            \
         0,                                                                         \
        "call address",                                                             \
        "rwlock",                                                                   \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x085, 0x08, INLINE, LK_WRUNLOCK, LK_WRUNLOCK,                       \
        //0x085 event
        "write_unlock",                                                             \
         0,                                                                         \
        "call address",                                                             \
        "rwlock",                                                                   \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x086, 0x08, INLINE, LK_RDLOCK, LK_RDLOCK,                           \
        //0x086 event
        "read_lock",                                                                \
         0,                                                                         \
        "call address",                                                             \
        "rwlock",                                                                   \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x087, 0x08, INLINE, LK_RDUNLOCK, LK_RDUNLOCK,                       \
        //0x087 event
        "read_unlock",                                                              \
         0,                                                                         \
        "call address",                                                             \
        "rwlock",                                                                   \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x088, 0x08, UNDEF,  LK_RESV88, LK_RESV88, NULL, 0, NULL, NULL, NULL, NULL)
LKST_ETYPE_DEF(0x089, 0x08, UNDEF,  LK_RESV89, LK_RESV89, NULL, 0, NULL, NULL, NULL, NULL)
LKST_ETYPE_DEF(0x08a, 0x08, UNDEF,  LK_RESV8a, LK_RESV8a, NULL, 0, NULL, NULL, NULL, NULL)
LKST_ETYPE_DEF(0x08b, 0x08, UNDEF,  LK_RESV8b, LK_RESV8b, NULL, 0, NULL, NULL, NULL, NULL)
LKST_ETYPE_DEF(0x08c, 0x08, UNDEF,  LK_RESV8c, LK_RESV8c, NULL, 0, NULL, NULL, NULL, NULL)
LKST_ETYPE_DEF(0x08d, 0x08, UNDEF,  LK_RESV8d, LK_RESV8d, NULL, 0, NULL, NULL, NULL, NULL)
LKST_ETYPE_DEF(0x08e, 0x08, UNDEF,  LK_RESV8e, LK_RESV8e, NULL, 0, NULL, NULL, NULL, NULL)
LKST_ETYPE_DEF(0x08f, 0x08, UNDEF,  LK_RESV8f, LK_RESV8f, NULL, 0, NULL, NULL, NULL, NULL)

LKST_ETYPE_DEF(0x090, 0x09, INLINE, O_PORTIN, O_PORTIN,                             \
        //0x090 event
        "portin",                                                                   \
         0,                                                                         \
        "port address/bite-width",                                                  \
        "value",                                                                    \
        "call address",                                                             \
         NULL)
LKST_ETYPE_DEF(0x091, 0x09, INLINE, O_PORTOUT, O_PORTOUT,                           \
        //0x091 event
        "portout",                                                                  \
         0,                                                                         \
        "port address/bite-width",                                                  \
        "value",                                                                    \
        "call address",                                                             \
         NULL)
LKST_ETYPE_DEF(0x092, 0x09, NORMAL, O_PANIC, O_PANIC,                               \
        //0x092 event
        "panic",                                                                    \
         0,                                                                         \
        "address of argument",                                                      \
        "call address",                                                             \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x093, 0x09, NORMAL, O_PRINTK, O_PRINTK,                             \
        //0x093 event
        "printk",                                                                   \
         0,                                                                         \
        "address of argument",                                                      \
        "call address",                                                             \
         NULL,                                                                      \
         NULL)

LKST_ETYPE_DEF(0x0a0, 0x0a, INLINE, TIMER_RUN, TIMER_RUN,                           \
        //0x0a0 event
        "timer_run",                                                                \
         0,                                                                         \
        "function address(fn)",                                                     \
        "parameter(data)",                                                          \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0x0a1, 0x0a, NORMAL, TIMER_ADD, TIMER_ADD,                           \
        //0x0a1 event
        "timer_add",                                                                \
         0,                                                                         \
        "timer",                                                                    \
        "timer->expires",                                                           \
        "timer->function",                                                          \
        "timer->data")
LKST_ETYPE_DEF(0x0a2, 0x0a, NORMAL, TIMER_MOD, TIMER_MOD,                           \
        //0x0a2 event
        "timer_mod",                                                                \
         0,                                                                         \
        "timer",                                                                    \
        "timer->expires",                                                           \
        "timer->function",                                                          \
        "timer->data")
LKST_ETYPE_DEF(0x0a3, 0x0a, NORMAL, TIMER_DEL, TIMER_DEL,                           \
        //0x0a3 event
        "timer_del",                                                                \
         0,                                                                         \
        "timer",                                                                    \
        "timer->expires",                                                           \
        "timer->function",                                                          \
        "timer->data")
#ifdef CONFIG_SMP
LKST_ETYPE_DEF(0x0a4, 0x0a, NORMAL, TIMER_DEL_SYNC, TIMER_DEL_SYNC,                 \
        //0x0a4 event
        "timer_del_sync",                                                           \
         0,                                                                         \
        "timer",                                                                    \
        "timer->expires",                                                           \
        "timer->function",                                                          \
        "timer->data")
#endif /* CONFIG_SMP */

LKST_ETYPE_DEF(0xF00, 0xF0, INLINE, LKST_INIT, LKST_INIT,                           \
        //0xf00 event
        "lkst_init",                                                                \
         0,                                                                         \
        "data",                                                                     \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)

LKST_ETYPE_DEF(0xF08, 0xF0, INLINE, LKST_MSET_XCHG, LKST_MSET_XCHG,                 \
        //0xf08 event
        "lkst_mset_xchg",                                                           \
         0,                                                                         \
        "old_id/new_id",                                                            \
        "old_maskset",                                                              \
        "new_maskset",                                                              \
         NULL)

LKST_ETYPE_DEF(0xF10, 0xF1, INLINE, LKST_BUFF_SHIFT, LKST_BUFF_SHIFT,               \
        //0xf10 event
        "lkst_buff_shift",                                                          \
         0,                                                                         \
        "buffer",                                                                   \
        "next_buffer",                                                              \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0xF11, 0xF1, INLINE, LKST_BUFF_OVFLOW, LKST_BUFF_OVFLOW,             \
        //0xf11 event
        "lkst_buff_ovflow",                                                         \
         0,                                                                         \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0xF12, 0xF1, UNDEF,  LKST_BUFF_HIGH, LKST_BUFF_HIGH,                 \
        //0xf12 event
        "lkst_buff_high",                                                           \
         0,                                                                         \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)

LKST_ETYPE_DEF(0xF18, 0xF1, UNDEF,  LKST_SYNC_TIME, LKST_SYNC_TIME,                 \
        //0xf18 event
        "lkst_sync_time",                                                           \
         0,                                                                         \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL,                                                                      \
         NULL)
LKST_ETYPE_DEF(0xF19, 0xF1, INLINE, LKST_SYNC_UID, LKST_SYNC_UID,                   \
        //0xf19 event
        "lkst_sync_uid",                                                            \
         0,                                                                         \
        "ruid",                                                                     \
         NULL,                                                                      \
        "currnet",                                                                  \
         NULL)
LKST_ETYPE_DEF(0xF1A, 0xF1, INLINE, LKST_SYNC_GID, LKST_SYNC_GID,                   \
        //0xf1a event
        "lkst_sync_gid",                                                            \
         0,                                                                         \
        "rgid",                                                                     \
         NULL,                                                                      \
        "currnet",                                                                  \
         NULL)
LKST_ETYPE_DEF(0xF1B, 0xF1, INLINE, LKST_SYNC_PGID, LKST_SYNC_PGID,                 \
        //0xf1b event
        "lkst_sync_pgid",                                                           \
         0,                                                                         \
        "pid",                                                                      \
        "pgrp",                                                                     \
        "pointer to task_struct",                                                   \
        "leader")

LKST_ETYPE_DEF(0xFFC, 0xFF, UNDEF,  LKST_EXTEND, LKST_EXTEND, NULL, 0, NULL, NULL, NULL, NULL)	// TODO: Tantative code
LKST_ETYPE_DEF(0xFFD, 0xFF, UNDEF,  LKST_EXTENDE, LKST_EXTENDE, NULL, 0, NULL, NULL, NULL, NULL)	// TODO: Tantative code
LKST_ETYPE_DEF(0xFFE, 0xFF, UNDEF,  LKST_OVWRTN_REC, LKST_OVWRTN_REC,               \
        //0xffe event
        "overwritten_record",                                                       \
         0,                                                                         \
        "old arg1",                                                                 \
        "old arg2",                                                                 \
        "old arg3",                                                                 \
        "old event_type")


#if	((!defined(LKST_ETYPE_MAX_IS_ENUM))&&(!defined(LKST_ETYPE_MAX)))
        /* By default, LKST_ETYPE_MAX is
           defined as an element of the enum.
           Otherwise here LKST_ETYPE_MAX is 
           defined as a MACRO.               */
#define LKST_ETYPE_MAX  (0xFFF+0)

#endif /* if ((!defined(LKST_ETYPE_MAX_IS_ENUM))&&(!defined(LKST_ETYPE_MAX))) */

#ifndef	LKST_ETYPE_VOID
#define LKST_ETYPE_VOID  (LKST_ETYPE_MAX+1)
#endif /* ifndef LKST_ETYPE_VOID */

    /* tail */
LKST_ETYPE_DEF_ENUM_END

        /* By default, undef LKST_ETYPE_DEF* 
           for the next inclusion of this file. */
#ifdef  LKST_ETYPE_DEF_TOCLEAN

#undef  LKST_ETYPE_DEF
#undef  LKST_ETYPE_DEF_ENUM_BEGIN
#undef  LKST_ETYPE_DEF_ENUM_END
#undef  LKST_ETYPE_DEF_TOCLEAN

#endif /* ifdef LKST_ETYPE_DEF_TOCLEAN */

/* END */
