/* $Id: lkst_private.h,v 1.7 2002/06/28 11:27:56 indoh Exp $ */
/* linux/include/linux/lkst_private.h */

/************************************************************
 *
 * COPYRIGHT (C) HITACHI,LTD. 2001-2002 ALL RIGHTS RESERVED.
 * WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
 *            HITACHI CENTRAL RESEARCH LABORATORY.
 *
 * Created by T.Nakamura <tetsu@crl.hitachi.co.jp>
 *
 ************************************************************/

#ifndef _LINUX_LKST_PRIVATE_H
#define _LINUX_LKST_PRIVATE_H


#ifndef CMD_LKSTDUMP
#include <linux/sched.h>
#include <linux/lkst.h>
#endif /* ifndef CMD_LKSTDUMP */
#include <linux/lkst_maskset.h>
#include <linux/lkst_buffer.h>
#include <linux/lkst_evhandler.h>
#ifndef CMD_LKSTDUMP
#include <linux/kernel_stat.h>
#include <linux/string.h>
#endif /* ifndef CMD_LKSTDUMP */
#include <linux/wait.h>
#include <linux/cache.h>

#include <asm/atomic.h>
#include <asm/lkst_private.h>

/* recid for CPU */
typedef int lkst_cpu_recid;
#define LKST_CPU_RECID_SIZE SMP_CACHE_BYTES/sizeof(lkst_cpu_recid)
#define LKST_CPU_RECID_BUF_SIZE LKST_CPU_MAX*LKST_CPU_RECID_SIZE

/* buffer size -- must be power of 2 */
#define LKST_BUFFER_SIZE_DEFAULT	(64*1024)	/* 64KBytes*/
#define LKST_BUFFER_SIZE_MAX		(64*1024*1024)	/* 64MBytes*/
#define LKST_BUFFER_SIZE_MIN		8192		/* 8KBytes */

	/* Rooms for overrun in buffer wrapping around.	
		   It assumes the worst case that all the CPUs 
		   interrupt in detection of buffer wrapping,
		   and exception occurs on each CPU.		*/
#define LKST_BUFFER_HEADSPACE						\
	(sizeof (struct lkst_event_record) * LKST_CPU_MAX * 2)

#define LKST_EVHANDLER_BUFFER_TRIAL_MAX		10
#define LKST_EVHANDLER_MASKSET_TRIAL_MAX	10

#define LKST_DEV_MAJOR	231	/* LKST device major number */  /* TODO */
#define LKST_DEV_MINOR	0	/* LKST device minor number */
#define LKST_DEV_NAME	"lkst"	/* LKST device name */

/* LKST buffer version */
#define LKST_BUF_VER	0x000100	/* major(1B) miner(1B) sub(1B) */

#define LKST_FLAG_YES	1
#define LKST_FLAG_NO	0

typedef lkst_evhandler_id *lkst_maskset;

struct lkst_maskset_table {
	char name[LKST_MASKSET_NAME_LEN];
	lkst_maskset *masksetp;
	volatile lkst_pointer_check_digit masksetp_check;
};


	/* Size of lkst_event_record is defined in <linux/lkst_buffer.h>, 
	   as LKST_SIZEOF_LKST_EVENT_RECORD. It must be power of 2.       */
	/* This value will be verified in initialization of 
	   LKST in DEBUG mode. */
struct lkst_event_record {
	int log_recid;
	int log_event_type;
	lkst_tsc_t log_time;
	pid_t log_pid;
  	int _reserved0;	     /* Reserved for cpu id - in case shared buffer. */
	lkst_arg_t log_arg1;
	lkst_arg_t log_arg2;
	lkst_arg_t log_arg3;
	lkst_arg_t log_arg4;
	lkst_arg_t _reserved1;
};

typedef void (*lkst_evhandler_func) (void *phookrec, int event_type,
				     lkst_arg_t arg1,
				     lkst_arg_t arg2,
				     lkst_arg_t arg3, lkst_arg_t arg4);

typedef int (*lkst_evhandler_ctrl_func) (void *buf, size_t bufsize);

struct lkst_evhandler_entry {
	char name[LKST_EVHANDLER_NAME_LEN];
	volatile lkst_evhandler_func evhandler;
	volatile lkst_pointer_check_digit evhandler_check;
	volatile lkst_evhandler_ctrl_func evhandler_ctrl;
	volatile lkst_pointer_check_digit evhandler_ctrl_check;
};

struct lkst_elapsed_times_buff {
	time_t current_time;
	atomic_t count;
	int count0;
	/* average, maximum, minimum and varianceof elapsed times */
	time_t avg_time;
	time_t max_time;
	time_t min_time;
	time_t var_time;
	/* room for optional times */
	time_t o0_time;
	time_t o1_time;
	time_t o2_time;
	time_t o3_time;

	/* room for optional event_types */
	int event_type0;
	int event_type1;
	int event_type2;
	int event_type3;
};

struct lkst_event_buffer {
	atomic_t writep;
	size_t size;			/* buffer size */
	size_t readp;
	lkst_buffer_id id, prev, next;	/* buffer ID of own/prev/next buffer */
	char _pad1;
	char eyecatcher[4];		/* EyeCacher */
	int cpu;			/* cpu number of the buffer */
	int last_recid;			/* recid of event record read last */
	size_t last_readp;		/* last readp */
	struct timeval xtime;		/* xtime */
	unsigned int cmos_time_flag; 	/* cmos time flag */
	int buf_ver;			/* LKST buffer version */
	atomic_t rcnt, wcnt;
	char _pad2[8];
	struct lkst_event_record body[LKST_BUFFER_SIZE_DEFAULT /
				      sizeof(struct lkst_event_record)];
					/* buffer body */
};

	//TODO : header size is changed by the architecture.
struct lkst_header {
	char eyecacher[4];	/* EyeCacher */
	atomic_t refcnt;	/* Reference counter of LKST. */

	/* recid for CPU */
	lkst_cpu_recid cpu_recid[LKST_CPU_RECID_BUF_SIZE];

	/* Current Maskset Pointer */
	volatile lkst_maskset *current_masksetp;
	volatile lkst_pointer_check_digit current_masksetp_check;
	volatile lkst_maskset_id current_maskset_id;
	/* Needed to be volatile, because the order of modification
	 *   and reference of these values are significant.
	 */

	/* Maskset Table */
	struct lkst_maskset_table maskset_table[LKST_MASKSET_TBL_MAX];

	/* Event Handler Table */
	struct lkst_evhandler_entry evhandler_table[LKST_EVHANDLER_TBL_MAX];

	/* IDs of current buffers. */
	lkst_buffer_id current_buffer_id[LKST_CPU_MAX];

	/* Event Buffer Table */
	struct lkst_buffer_table {
				// TODO: volatile struct lkst_event_buffer *bufferp;
		struct lkst_event_buffer *bufferp;
		/* event_buffer address */
		lkst_pointer_check_digit bufferp_check;
	} buffer_table[LKST_BUFFER_TBL_MAX];


	/* Below variables are needed for all buffer list.
	 * In current implementation, each Buffer list is corresponded to CPU.
	 */
	/* Tail of Buffer list */
	lkst_buffer_id buffer_tail[LKST_CPU_MAX];
	/* Head of Buffer list */
	lkst_buffer_id buffer_head[LKST_CPU_MAX];
	/* ring structure flag */
	unsigned int buffer_ring[LKST_CPU_MAX];
	/* owner pid of Buffer list */
	pid_t buffer_owner[LKST_CPU_MAX];
	/* ready to read (wakeup) flag */
	int owner_ready[LKST_CPU_MAX];
	/* buffer wait queue */
	wait_queue_head_t buffer_wq[LKST_CPU_MAX];

	/* Stopped maskset id */
	lkst_maskset_id maskset_id_saved;

	/* LKST running status flag */
	atomic_t status_flag;

	/* LKST available memory size */
	size_t available_memory_size;
	/* LKST available memory limit address */
	void *limit_addr;

	int endian_big;			/* byte order, 0 if little endian */
	lkst_cpu_freq_t	cpu_freq;	/* cpu clockspeed in kHz */
	lkst_tsc_t tsc;			/* machine cycle */
	struct timeval xtime;		/* xtime */
	unsigned int cmos_time_flag; 	/* cmos time flag */
	int buf_ver;			/* LKST buffer version */
	char arch[LKST_ARCH_NAME_LEN];	/* Architecture name */
};

#ifdef CONFIG_DEBUG_KERNEL_STATE_TRACE_MODULE
extern struct lkst_header *get_lkst(void);
#endif /* CONFIG_DEBUG_KERNEL_STATE_TRACE_MODULE*/

extern __inline__ unsigned long lkst_get_cpu_khz(void);

#ifndef CMD_LKSTDUMP
int lkst_open(struct inode *inode, struct file *filp);

int lkst_close(struct inode *inode, struct file *filp);

ssize_t lkst_read(struct file *filp, char *buf, size_t count, loff_t *ppos);

int lkst_ioctl(struct inode *inode, struct file *filp,
	       unsigned int request, unsigned long arg);

extern int lkst_evhandler_register(lkst_evhandler_id id, 
				   const char *name,
				   lkst_evhandler_func evhandler,
				   lkst_evhandler_ctrl_func
				   evhandler_ctrl);
extern int lkst_evhandler_unregister(lkst_evhandler_id id);
extern int lkst_evhandler_get_id(const char *name);

extern void lkst_buffer_dump_current_txt(void);

extern int lkst_evhandlerprim_recid_inc(void);
extern struct lkst_event_buffer
*lkst_evhandlerprim_select_buffer_cpu(int cpu);
extern struct lkst_event_buffer
*lkst_evhandlerprim_select_buffer_id(lkst_buffer_id id);
extern struct lkst_event_buffer
*lkst_evhandlerprim_select_buffer(void);
extern struct lkst_event_record
*lkst_evhandlerprim_get_write_entry(int event_type);
extern struct lkst_event_buffer
*lkst_evhandlerprim_getw_buffer_cpu(int cpu);
extern struct lkst_event_buffer
*lkst_evhandlerprim_getw_buffer(void);
extern void
lkst_evhandlerprim_putw_buffer(struct lkst_event_buffer *buffer);
extern void lkst_evhandlerprim_daemon_wakeup(int cpu);
extern void lkst_evhandlerprim_entry_log_l2(int event_type,
					    lkst_arg_t arg1, lkst_arg_t arg2,
					    lkst_arg_t arg3, lkst_arg_t arg4);

/* System default event handlers */

extern void lkst_evhandler_default(void *phookrec, int event_type,
				   lkst_arg_t arg1,
				   lkst_arg_t arg2,
				   lkst_arg_t arg3, lkst_arg_t arg4);

extern int lkst_evhandler_default_ctrl(void *buf, size_t buf_size);

extern void lkst_evhandler_buffer_shift(void *phookrec, int event_type,
					lkst_arg_t arg1,
					lkst_arg_t arg2,
					lkst_arg_t arg3,
					lkst_arg_t arg4);

extern void lkst_evhandler_buffer_shift_dw(void *phookrec, int event_type,
					   lkst_arg_t arg1,
					   lkst_arg_t arg2,
					   lkst_arg_t arg3,
					   lkst_arg_t arg4);

extern void lkst_evhandler_maskset_xchg(void *phookrec, int event_type,
					lkst_arg_t arg1,
					lkst_arg_t arg2,
					lkst_arg_t arg3,
					lkst_arg_t arg4);

#if DEBUG
extern void lkst_evhandler_exception(void *phookrec, int event_type,
				     lkst_arg_t arg1,
				     lkst_arg_t arg2,
				     lkst_arg_t arg3,
				     lkst_arg_t arg4);
#endif /* DEBUG */

#endif /* ifndef CMD_LKSTDUMP */


				/* entry address indexed in buffer body */
#define LKST_BUFFER_BODY_INDEX(buffer,index)				\
	(&(((struct lkst_event_buffer *)(buffer))->body[(index)]))

				/* Top of buffer body */
#define LKST_BUFFER_BODY_TOP(buffer)					\
	(LKST_BUFFER_BODY_INDEX((buffer),0))

				/* Offset of buffer body from buffer head */
#define LKST_BUFFER_BODY_OFFSET()					\
	((size_t)LKST_BUFFER_BODY_TOP(NULL))

				/* size of the buffer whole */
#define LKST_BUFFER_AREA_SIZE(n)					\
	(LKST_BUFFER_BODY_OFFSET() + (n))

#ifdef LKST_DEBUG_PRINTK
#define DPRINTK(fmt, args...)						\
	printk(KERN_ERR "LKST(%s): " fmt, __FUNCTION__, ## args)
#else
#define DPRINTK(fmt, args...)
#endif


#ifndef CMD_LKSTDUMP

// TODO
# ifdef CONFIG_MODULES

extern char _stext, _etext;
extern struct module *module_list;
extern struct module kernel_module;

/*
 * function lkst_find_module_by_address
 * Find module in which specified address is contained, and return
 * pointer to the module.
 * 
 *  Caution: This function does not verify value of event_entry_p.
 * 
 *  Arguments
 *    addr {IN}:	address.
 *
 *  returns
 *    address to the module.
 * 
 */
static __inline__ struct module *
lkst_find_module_by_address_inline(unsigned long addr)
{
	struct module *mod;

	if (addr >= (unsigned long) &_stext &&
	    addr <= (unsigned long) &_etext)
		return &kernel_module;

	// TODO: avoid conflicts without spin_lock(&modlist_lock);

	for (mod = module_list;
	     mod != &kernel_module; mod = mod->next) {
		if (!(mod->flags&MOD_RUNNING))
			return NULL;
		if (mod->flags&MOD_DELETED)
			return NULL;
		/* mod_bound tests for addr being inside the vmalloc'ed
		 * module area. Of course it'd be better to test only
		 * for the .text subset... */
		if (mod_bound(addr, 0, mod)) {
			return mod;
			break;
		}
	}
	/* address was not found. */
	return NULL;
}
extern struct module * lkst_find_module_by_address(unsigned long addr);
#endif /* ifdef CONFIG_MODULES */


/*
 * function lkst_evhandlerprim_entry_next()
 * A primitive part of an Event-handler. This function write specified record
 * to specified entry.
 * 
 *  Caution: This function does not verify value of event_entry_p.
 * 
 *  Arguments
 *    buffer {OUT}:	address of control block of the buffer to write.
 *    event_type {IN}:	corresponding event type.
 *
 *  returns
 *    event_entry_p:	address of entry to write.
 *      NULL		found no entry to write.
 * 
 */
static __inline__ struct lkst_event_record
*lkst_evhandlerprim_entry_next(struct lkst_event_buffer *buffer,
			       int event_type)
{
	struct lkst_event_record *event_entry_p;
	size_t writep;
	size_t sizem = buffer->size - 1;//TODO:

	writep = ((size_t)(buffer->writep.counter)) & sizem;
	buffer->writep.counter = writep + sizeof(struct lkst_event_record);

				/* Check buffer wrapping. */
	if (!writep) {
				/* Wrapped. */
		if (event_type == LKST_ETYPE_LKST_BUFF_OVFLOW) {
				/* To avoid infinite loop, retrn NULL
				   then caller do nothing.		*/
			return NULL;
		}
		else {
				/* Process buffer-over-flow event. */
			lkst_entry_log(NULL, LKST_ETYPE_LKST_BUFF_OVFLOW,
				       buffer->cpu,
				       buffer->next,
				       event_type,
				       LKST_ARGP(buffer));
		}
	}

	(char *) event_entry_p =
		(char *)LKST_BUFFER_BODY_TOP(buffer) + writep;

	return (event_entry_p);
}

/*
 * function lkst_evhandlerprim_entry()
 * A primitive part of an Event-handler. This function write specified record
 * to specified entry.
 * 
 *  Caution: This function does not verify value of event_entry_p.
 * 
 *  Arguments
 *    buffer {OUT}:	address of control block of the buffer to write.
 *    event_type {IN}:	corresponding event type.
 *    index {IN}:	index of the entry to write.
 *
 *  returns
 *    event_entry_p:	address of entry to write.
 *      NULL		found no entry to write.
 * 
 */
static __inline__ struct lkst_event_record
*lkst_evhandlerprim_entry(struct lkst_event_buffer *buffer,
			  int event_type, int index)
{
	struct lkst_event_record *event_entry_p;

	if (((index + 1) * sizeof(struct lkst_event_record))
	    > buffer->size)
		return NULL;

	event_entry_p = LKST_BUFFER_BODY_INDEX(buffer,index);

	return (event_entry_p);
}

/*
 * function lkst_evhandlerprim_entry_log_l1
 * A primitive part of an Event-handler. This function write specified record
 * to specified entry.
 * 
 *  Caution: This function does not verify value of event_entry_p.
 * 
 *  Arguments
 *    event_entry_p {OUT}:
 *			address of the entry to which the record 
 *			will be written.
 *    recid {IN}:	ID of event record to be recorded.
 *    event_type {IN}:	corresponding event type.
 *    arg1-4 {IN}:	arguments acquired at a trace point
 *
 *  returns
 *    NONE
 * 
 */
static __inline__ void lkst_evhandlerprim_entry_log_l1(struct lkst_event_record
						   *event_entry_p, int recid,
						   int event_type,
						   lkst_arg_t arg1,
						   lkst_arg_t arg2,
						   lkst_arg_t arg3,
						   lkst_arg_t arg4)
{
	event_entry_p->log_time = lkst_evhandlerprim_mc(); /* current time */

	event_entry_p->log_event_type = event_type;
	event_entry_p->log_pid = current->pid; /* current pid */ ;
	event_entry_p->log_arg1 = arg1;
	event_entry_p->log_arg2 = arg2;
	event_entry_p->log_arg3 = arg3;
	event_entry_p->log_arg4 = arg4;
	event_entry_p->log_recid = recid;
}

// TODO
#ifdef CONFIG_MODULES
/*
 * function lkst_evhandlerprim_kernel_text_address
 * A primitive part of an Event-handler. This returns whether
 * addr is in kernel (or kernel modules) text address.
 *
 *  Arguments
 *    addr {IN}: address to investigate.
 *
 *  returns
 *    0: addr is out of the kernel code nor kernel module.
 *    1: addr is in the kernel code or kernel module.
 *
 */
static __inline__ int lkst_evhandlerprim_kernel_text_address(unsigned long addr)
{
	return (NULL != lkst_find_module_by_address(addr));
}
#endif /*#ifdef CONFIG_MODULES */

#endif /* ifndef CMD_LKSTDUMP */


#define LKST_ETYPE_VERIFY(event_type) 					\
	(!(0xfffff000&(event_type)))

			/*  p + p##_check == LKST_POINTER_CHECK_MAGIC */
#define LKST_POINTER_CHECK_VAL(p) 					\
	(LKST_POINTER_CHECK_MAGIC - ((lkst_pointer_check_digit)(p)))
			/* initialize p##_check */
#define LKST_POINTER_CHECK_INIT(p) 					\
	(p##_check = LKST_POINTER_CHECK_VAL(p))
			/* copy NULL into p and p##_check*/
#define LKST_POINTER_INVALIDATE(p)					\
	(p##_check = (lkst_pointer_check_digit)(0), (void *)(p) = NULL )
			/* verify p and p##_check */
#define LKST_POINTER_VERIFY_STRICT(p1, p2)				\
	((p2) = (p1),							\
	 (void *)(p1##_check) == (void *)LKST_POINTER_CHECK_VAL((p2)))
#define LKST_POINTER_VERIFY(p1, p2) LKST_POINTER_VERIFY_STRICT(p1, p2)
#define LKST_POINTER_VERIFY_HP(p1, p2)					\
	((p2) = (p1), 1)

#define	LKST_MASKSET_QUERY_EVHANDLER_ID(maskset_ptr, event_type)	\
	(((lkst_evhandler_id *)(maskset_ptr))[(event_type)])
#define	LKST_MASKSET_SET_EVHANDLER_ID(maskset_ptr, event_type, id)	\
	(((lkst_evhandler_id *)(maskset_ptr))[(event_type)] = (id))

#define	LKST_MASKSET_TABLE_ENTRY_SET(table, id, maskname, addr) ({     	\
	(table)[(id)].masksetp = (lkst_maskset *)(addr);		\
	strncpy((table)[(id)].name, (maskname),				\
		(size_t) LKST_MASKSET_NAME_LEN);			\
	(table)[id].name[LKST_MASKSET_NAME_LEN-1] = '\0';		\
	LKST_POINTER_CHECK_INIT((table)[(id)].masksetp); })

#define	LKST_MASKSET_SET_CURRENT(id) ({					\
	lkst.current_maskset_id = (id);					\
	lkst.current_masksetp =						\
		lkst.maskset_table[(id)].masksetp;			\
	LKST_POINTER_CHECK_INIT(lkst.current_masksetp);})

#define LKST_BUFFER_PREV_ENTRY(buffer, pt, size)			\
	((struct lkst_event_record *) ((char *) (buffer) +		\
	(((pt) - sizeof(struct lkst_event_record)) & (size - 1))))

#define LKST_BUFFER_READP_RESET(buffer)	({				\
	(buffer)->readp = LKST_BUFFER_HEADSPACE;			\
	(buffer)->last_readp =						\
		((buffer)->readp - sizeof(struct lkst_event_record)) &	\
	   	((buffer)->size - 1);					\
	(buffer)->last_recid =						\
		((struct lkst_event_record *)				\
		((char *) ((buffer)->body)				\
			 + (buffer)->last_readp))->log_recid;})

#endif /* _LINUX_LKST_PRIVATE_H */
